<?php
if (! defined('ABSPATH') ) {
	exit; // Exit if accessed directly
}

/**
 * Class to Apprear Wheel Popup on Site
 */
class Woo_Wheel_Events extends Woo_Wheel_View {



	public function __construct() {
		add_filter('woocommerce_registration_redirect', array( $this, 'after_user_login' ), 50, 1);
		add_filter('woocommerce_login_redirect', array( $this, 'login_redirect' ), 10, 2);
		add_action('wp_footer', array( $this, 'display_wheel' ), 10, 1);
		add_action('wp_ajax_generating_coupon', array( $this, 'generating_coupon_ajax' ));
		add_action('wp_ajax_nopriv_generating_coupon', array( $this, 'generating_coupon_ajax' ));
		add_action('wp_ajax_generated_coupon', array( $this, 'generated_coupon_ajax' ));
		add_action('wp_ajax_nopriv_generated_coupon', array( $this, 'generated_coupon_ajax' ));
		add_action('woocommerce_add_to_cart', array( $this, 'product_added_to_cart' ));
		add_action('woocommerce_before_cart', array( $this, 'woo_wheel_apply_coupon' ));
		add_action('woocommerce_thankyou', array( $this, 'remove_coupon_cookie' ), 10, 1);
		add_action('wp_ajax_woo_spin_wheel', array( $this, 'woo_spin_wheel_callback' ));
		add_action('wp_ajax_nopriv_woo_spin_wheel', array( $this, 'woo_spin_wheel_callback' ));
		add_action('woocommerce_init', array( $this, 'woo_wheel_assign_coupon_to_winner_user' ), 10, 3);
		add_action('woocommerce_before_calculate_totals', array( $this, 'woo_wheel_disallow_coupon_for_non_logged_in' ), 50);
	}

	public function product_added_to_cart() {
		
		add_action('wp_footer', array( $this, 'render_wheel_on_add_cart_button' ));
	}

	public function render_wheel_on_add_cart_button() {
		if (!wp_doing_ajax() ) {
			?>
				<script type="text/javascript">
					// Ready state
					if ( typeof woo_wheel_open === "function" )
						woo_wheel_open();
					else
						var wheel_triggered = 1;
				</script>
			<?php
		}
	}

	/**
	 * Display Wheel on site Frontend
	 *      
	 * @since   1.0
	 * @version 1.1
	 */
	public function display_wheel() {

		$this->snowfall();
		// Check Wheel have slices or not
		$no_wheel_slices = woo_wheel_get_slices_count();
		if (0 == $no_wheel_slices) {
			return;
		}

		$display_wheel     = false;
		$user_id           = get_current_user_id();
		$thank_page        = woo_wheel_get_option('woo_wheel_settings_thankyou_page');
		$after_add_to_cart = woo_wheel_get_option('woo_wheel_settings_after_add_to_cart_event');

		// If after add to cart event is enabled from setting
		if ('yes' == $after_add_to_cart && is_user_allowed_to_play($user_id)) {
			$display_wheel = true;
			$this->woo_wheel_html();
			// if the theme is block
			$theme = wp_get_theme(); 
			if ($theme->is_block_theme()) {

				?>
				 <script type="text/javascript">
					// Ready state
					(function($) {
						$(document.body).on('click', '.add_to_cart_button, .single_add_to_cart_button', function(e) {
							let timeout = 4000;
							setTimeout(() => {
								woo_wheel_open();
							}, timeout);
						});
					})(jQuery);
				</script>
				<?php
			} else {
				?>
				<script type="text/javascript">
					// Ready state
					(function($) {
						$(document.body).on('added_to_cart', function(event, fragments, cart_hash, $thisbutton)  {
							woo_wheel_open();
						});
					})(jQuery);
				</script>
				<?php
			}
		}

		// Check if user can play the spin or not depending on occurrence delay
		if (is_user_allowed_to_play($user_id)) {
			$display_wheel = true;
		}

		// filter to modify display wheel condition
		/**
		 * Filter the display of the Woo Wheel plugin.
		 *
		 * This filter allows developers to modify whether or not the Woo Wheel plugin is displayed on the frontend of the site.
		 *
		 * @param  bool $display_wheel Whether or not to display the Woo Wheel plugin. Default is true.
		 * @return bool Whether or not to display the Woo Wheel plugin.
		 *
		 * @since 1.0.0
		 */
		$display_wheel = apply_filters('woo_wheel_display', $display_wheel);

		// Condition to show wheel on thank you page.
		if (is_wc_endpoint_url('order-received') && 'yes' == $thank_page) {
			$display_wheel = true;
			$this->woo_wheel_html();
			?>
			<script type="text/javascript">
				// Ready state
				(function($) {
					woo_wheel_open();
				})(jQuery);
			</script>
			<?php
		}

		if (isset($_GET['registration_wheel'])) {
			$after_signup    = woo_wheel_get_option('woo_wheel_settings_after_signup');
			$woo_wheel_login = get_user_meta($user_id, 'woo_wheel_login', true);
			if ('yes' == $after_signup && 1 != $woo_wheel_login) {
				update_user_meta($user_id, 'woo_wheel_login', 1);
				$display_wheel = true;
			}
		}

		if (isset($_GET['woo_wheel']) && current_user_can('manage_options')) {
			$display_wheel = true;
			$this->woo_wheel_html();
			?>
				<script type="text/javascript">
					// Ready state
					(function($) {
						woo_wheel_open();
					})(jQuery);
				</script>
			<?php
		}

		// If Wheel tap is enabled
		if (is_user_allowed_to_play() && is_wheel_allow_on_pages_posts()) {
			$this->woo_wheel_html();
			?>
				<script type="text/javascript">
					// Ready state
					(function($) {
						woo_wheel_open();
					})(jQuery);
				</script>
			<?php
		}

		if (isset($_GET['registration_wheel']) && is_user_allowed_to_play($user_id)) {
			if (true == $display_wheel) {
				$this->woo_wheel_html();
				?>
				<script type="text/javascript">
					// Ready state
					(function($) {
						woo_wheel_open();
					})(jQuery);
				</script>
				<?php
			}
		}
	}

	/**
	 * Redirect user to my account page after login with a query string for open a wheel
	 *
	 * @param   $redirect (string) URL to Redirect
	 * @since   1.0
	 * @version 1.0
	 */
	public function after_user_login( $redirect ) {

		$user_id         = get_current_user_id();
		$after_signup    = woo_wheel_get_option('woo_wheel_settings_after_signup');
		$woo_wheel_login = get_user_meta($user_id, 'woo_wheel_login', true);
		if ('yes' == $after_signup && 1 != $woo_wheel_login ) {
			return get_permalink(get_option('woocommerce_myaccount_page_id')) . '?registration_wheel';
		}
		return $redirect;
	}

	/**
	 * Redirect to shop after login.
	 *
	 * @param $redirect
	 * @param $user
	 *
	 * @return false|string
	 */
	public function login_redirect( $redirect, $user ) {

		if (isset($_GET['login_redirect']) ) {
			$redirect = sanitize_text_field(wp_unslash($_GET['login_redirect'])) . '?woo_wheel_prize';
		}

		return $redirect;
	}

	/**
	 * Ajax callback to generate dynamic coupon if a wheel stop on auto generated coupon
	 *      
	 * @since   1.0
	 * @version 1.0
	 */
	public function generating_coupon_ajax() {

		check_ajax_referer('coupon_gen', 'security');

		if (!empty($_POST['amount']) ) {
			$amount      = intval($_POST['amount']);
			$coupon_code = $this->grenerate_coupon($amount);
			$url = wc_get_cart_url();

			update_option('woo_wheel_get_coupon_code', $coupon_code);

			$this->set_coupon_cookie_expiry_time($coupon_code);
			

		}

		die();
	}

	/**
	 * Ajax callback to generated dynamic coupon if a wheel stop on generated coupon
	 *      
	 * @since   1.8
	 * @version 1.8
	 */
	public function generated_coupon_ajax() {
		check_ajax_referer('coupon_gen', 'security');
		if (isset($_POST['couponValue'])  ) {
			$coupon_code =  sanitize_text_field(wp_unslash($_POST['couponValue']));
			$url = wc_get_cart_url();

			update_option('woo_wheel_get_coupon_code', $coupon_code);
			
			$this->set_coupon_cookie_expiry_time($coupon_code);
		}

		die();
	}

	/**
	 * To set the coupon cookie expiry time
	 *      
	 * @since   1.8
	 * @version 1.8
	 */
	public function set_coupon_cookie_expiry_time( $coupon_code = '' ) {

		/**
		   * Filter to modify the cookie expiry time of the Wheel Coupon.
		   * 
		   * @param  int 7 The expiry time for cookie in days.
		   * @return int Modified cookie expiry time of the Wheel Coupon.
		   * @since  1.0.0
		   */
		$default_cookie_expiry_time = apply_filters('woo_wheel_expire_cookie_default_time', 7);
		$expiry_interval = !empty(woo_wheel_get_option('woo_wheel_settings_coupon_duration')) ? intval(woo_wheel_get_option('woo_wheel_settings_coupon_duration')) : $default_cookie_expiry_time;
		$expiry_time     = woo_wheel_get_option('woo_wheel_settings_coupon_duration_time');
		if ('hour' == $expiry_time ) {
			$secs = $expiry_interval * 3600; // 3600 = 1 hour
		} else {
			$secs = $expiry_interval * 86400; // 86400 = 1 day
		}
   
		setcookie('woo_wheel_gen_code', strtolower($coupon_code), time() + $secs, '/'); 
		echo esc_html($coupon_code);
	}

	public function woo_wheel_apply_coupon() {

		$add_coupon_to_cart_option = get_option('woo_wheel_settings_coupon_add_to_cart');
		$winning_coupon_code = get_option('woo_wheel_settings_winning_text_coupon');
		
		if ( 'yes' == $add_coupon_to_cart_option && !empty($winning_coupon_code) && is_page('cart') && is_user_logged_in() ) {

			if (isset($_COOKIE['woo_wheel_gen_code']) ) {
				$coupon_code = sanitize_text_field(wp_unslash($_COOKIE['woo_wheel_gen_code']));
			} else {
				$coupon_code = '';
			}
			
			if (empty($coupon_code) || !$coupon_code ) {
				return;
			}

			if (WC()->cart->has_discount($coupon_code) ) {
				return;
			}

			WC()->cart->apply_coupon($coupon_code);

			
			wc_print_notices();
		}
	}

	public function remove_coupon_cookie( $order_id ) {
		
		if (! $order_id ) {
			return;
		}

		// Allow code execution only once 
		$hposupdate_meta = wc_get_order($order_id);
		$get_post_meta=$hposupdate_meta->get_meta('_thankyou_action_done', true);
		//if ( ! get_post_meta( $order_id, '_thankyou_action_done', true ) ) { 
		if (! $get_post_meta ) {    
			// Get an instance of the WC_Order object
			//$order       = wc_get_order( $order_id );
			//$all_coupons = $order->get_coupon_codes();
			$all_coupons = $hposupdate_meta->get_coupon_codes();

			if (! empty($all_coupons) && isset($_COOKIE['woo_wheel_gen_code']) ) {
				// echo '<pre>';
				//     print_r($all_coupons);
				// echo '</pre>';

				foreach ( $all_coupons as $coupon_code ) {

					if ($coupon_code == $_COOKIE['woo_wheel_gen_code'] ) {            
						//remove cookie from browser
						setcookie('woo_wheel_gen_code', '', time() - 3600, '/');
					}
				}
			
			}
		}
	}

	/**
	 * Ajax callback to generate dynamic coupon if a wheel stop on auto generated coupon
	 *      
	 * @since   1.0
	 * @version 1.0
	 */
	public function grenerate_coupon( $amount ) {

		$coupon_code   = $this->generate_random_coupon_code(5); // Generating Random String
		$discount_type = 'percent'; // Type: fixed_cart, percent, fixed_product, percent_product
							
		$coupon = array(
		'post_title'    => wp_strip_all_tags($coupon_code),
		'post_content'  => '',
		'post_status'   => 'publish',
		'post_author'   => 1,
		'post_type'        => 'shop_coupon',
		);
							
		$new_coupon_id = wp_insert_post($coupon);

		$expiry_interval = woo_wheel_get_option('woo_wheel_settings_coupon_duration');
		$expiry_time     = woo_wheel_get_option('woo_wheel_settings_coupon_duration_time');
		$expiry_date     = gmdate('Y-m-d H:s:i', strtotime(' +' . $expiry_interval . ' ' . $expiry_time));
							
		// Add meta
		update_post_meta($new_coupon_id, 'discount_type', $discount_type);
		update_post_meta($new_coupon_id, 'coupon_amount', $amount);
		update_post_meta($new_coupon_id, 'individual_use', 'yes');
		update_post_meta($new_coupon_id, 'product_ids', '');
		update_post_meta($new_coupon_id, 'exclude_product_ids', '');
		update_post_meta($new_coupon_id, 'usage_limit', 1);
		update_post_meta($new_coupon_id, 'expiry_date', $expiry_date);
		update_post_meta($new_coupon_id, 'apply_before_tax', 'yes');
		update_post_meta($new_coupon_id, 'free_shipping', 'no');

		return $coupon_code;
	}

	/**
	 * Generate Random Coupon Code
	 * 
	 * @param   $length (int) length of a coupon code
	 * @since   1.0
	 * @version 1.0
	 */
	public function generate_random_coupon_code( $length = 10 ) {
		$characters       = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$charactersLength = strlen($characters);
		$randomString     = '';
		for ($i = 0; $i < $length; $i++) {
			$randomString .= $characters[rand(0, $charactersLength - 1)];
		}
		return $randomString;
	}
	public function woo_spin_wheel_callback() {
		$wheel_slices = woo_wheel_get_option('woo_wheel_slices');
		$sound        = woo_wheel_get_option('woo_wheel_settings_section_enable_sound');
		
		if (empty($wheel_slices) && ! is_array($wheel_slices) ) {
			return;
		}

		$wheel_attr  = array();
		$wheel_attr2 = array();
		$counter     = 0;
		// echo count( $wheel_slices['woo_wheel_slices']['text'] );
		for ( $i = 0; $i < count($wheel_slices['woo_wheel_slices']['text']); $i++ ) {

			if ('coupon' == $wheel_slices['woo_wheel_slices']['coupon_type'][$i] ) {
				$wheel_value = get_the_title($wheel_slices['woo_wheel_slices']['woo_coupon'][$i]); // Getting WooCommerce Coupon ID
			} elseif ('coupon_auto' == $wheel_slices['woo_wheel_slices']['coupon_type'][$i] ) {
				$wheel_value = $wheel_slices['woo_wheel_slices']['woo_coupon'][$i];
			} elseif ('link' == $wheel_slices['woo_wheel_slices']['coupon_type'][$i] ) {
				$wheel_value = $wheel_slices['woo_wheel_slices']['value'][$i];
			} elseif ('no_prize' == $wheel_slices['woo_wheel_slices']['coupon_type'][$i] ) {
				$wheel_value = $wheel_slices['woo_wheel_slices']['value'][$i];
			}

			//Check Wheel Slice Probability for Fixed Coupon
			$probability = $wheel_slices['woo_wheel_slices']['chance'][$i];
			if ('coupon' == $wheel_slices['woo_wheel_slices']['coupon_type'][$i] && false == woo_wheel_is_coupon_valid($wheel_value) ) {
				$probability = 0;
			}

			$wheel_attr[$counter]['probability'] = $probability;
			$wheel_attr[$counter]['type']        = 'string';
			$wheel_attr[$counter]['value']       = $wheel_slices['woo_wheel_slices']['text'][$i];
			$wheel_attr[$counter]['win']         = false;
			$wheel_attr[$counter]['couponType']  = $wheel_slices['woo_wheel_slices']['coupon_type'][$i];
			$wheel_attr[$counter]['resultText']  = $wheel_slices['woo_wheel_slices']['text'][$i];
			$wheel_attr[$counter]['resultValue'] = $wheel_value;
			$wheel_attr[$counter]['userData']    = array( 'score' => 10 );

			$counter++;
		}

		$border_color = woo_wheel_get_option('woo_wheel_settings_border_color');

		$data = array(
		'colorArray' => $wheel_slices['woo_wheel_slices']['bg_color'],
		'segmentValuesArray' => $wheel_attr,
		'svgWidth' => 1024,
		'svgHeight' => 768,
		'wheelStrokeColor' => $border_color,
		'wheelStrokeWidth' => 18,
		'wheelSize' => 700,
		'wheelTextOffsetY' => 80,
		'wheelTextColor' => '#EDEDED',
		'wheelTextSize' => '2.3em',
		'wheelImageOffsetY' => 40,
		'wheelImageSize' => 50,
		'centerCircleSize' => 200,
		'centerCircleStrokeColor' => $border_color,
		'centerCircleStrokeWidth' => 12,
		'centerCircleFillColor' => '#EDEDED',
		'segmentStrokeColor' => '#dc8dc7',
		'segmentStrokeWidth' => 4,
		'centerX' => 512,
		'centerY' => 384,  
		'hasShadows' => false,
		'numSpins' => -1,
		'spinDestinationArray' => array(),
		'minSpinDuration' => 6,
		'gameOverText' => '',
		'invalidSpinText' =>'INVALID SPIN. PLEASE SPIN AGAIN.',
		'introText' => '',
		'hasSound' => ( 'yes' == $sound ? true : false ),
		'gameId' => '9a0232ec06bc431114e2a7f3aea03bbe2164f1aa',
		'clickToSpin' => true,
		'spinDirection' => 'ccw',
		);        
		
		echo wp_json_encode($data);
		wp_die();
	}

	public function woo_wheel_assign_coupon_to_winner_user() {
		if (isset($_COOKIE['woo_wheel_gen_code']) && is_user_logged_in()) {
			$coupon_code = sanitize_text_field($_COOKIE['woo_wheel_gen_code']);
			$customer_id = get_current_user_id();
			// Retrieve user's existing coupons
			$user_coupons = get_user_meta($customer_id, 'whoo_wheel_user_coupons', true);
			// If user has existing coupons, add the new coupon to the array, otherwise create a new array with the coupon
			$user_coupons = !empty($user_coupons) ? $user_coupons : array();
			if (!in_array($coupon_code, $user_coupons)) {
				$user_coupons[] = $coupon_code; // Add new coupon to array
				$wc_coupon = new WC_Coupon($coupon_code);
				if ($wc_coupon->get_id()) {
					update_user_meta($customer_id, 'whoo_wheel_user_coupons', $user_coupons);
				}
			}
		}
	}

	public function woo_wheel_disallow_coupon_for_non_logged_in( $cart ) {
		// Check if the user is not logged in
		if (!empty($cart->get_coupons()) && !is_user_logged_in()) {
			$manual_coupons = get_option('woo_manual_coupons');

			foreach ( $cart->get_coupons() as $code => $coupon ) {
				$win_coupon = isset($_COOKIE['woo_wheel_gen_code']) ? sanitize_text_field($_COOKIE['woo_wheel_gen_code']) :'';
				if (( !empty($win_coupon)  && $win_coupon == $code ) || ( !empty($manual_coupons) && in_array($code, $manual_coupons) ) ) {
					$cart->remove_coupon($code);
					// Translators: %s is the coupon code.
					wc_add_notice(sprintf(__('Please sign in to apply "%s" coupon.', 'woocommerce'), $coupon->get_code()), 'error');
				}
			}
		}
	}
}

$Woo_Wheel_Events = new Woo_Wheel_Events();
