<?php
if (! defined('ABSPATH') ) {
	exit; // Exit if accessed directly
}

class Woo_Wheel_Settings_Tab {



	private $data = array();
	public $attachment_id = false;
	public $image_url = false;

	public function __construct() {

		$this->image_url = $this->get_type_image();

		add_filter('woocommerce_settings_tabs_array', array( $this, 'add_settings_tab' ), 50);
		add_action('woocommerce_settings_tabs_woo_wheel_settings', array( $this, 'settings_tab' ));
		add_action('woocommerce_update_options_woo_wheel_settings', array( $this, 'update_settings' ));
		add_action('woocommerce_admin_field_slices', array( $this, 'add_wheel_field_slices' ));
		add_action('woocommerce_admin_field_date_time', array( $this, 'add_wheel_field_date_time' ));

		// Clickable Button Image
		add_action('woocommerce_admin_field_image_upload', array( $this, 'add_wheel_image_upload' ));

		// Settings
		add_action('woocommerce_update_options', array( $this, 'save_slices' ));
		add_action('woocommerce_update_options', array( $this, 'save_image' ), 2000);
		add_action('woocommerce_update_options', array( $this, 'save_limited_time_wheel' ));
		add_action('admin_enqueue_scripts', array( $this, 'admin_scripts_styles' ));
		add_action('wp_enqueue_scripts', array( $this, 'frontend_scripts_styles' ));
		add_action('woo_wheel_setting_tab_preview', array( $this, 'setting_tab_preview' ));
		add_action('wp_ajax_remove_image', array( $this, 'remove_image' ));
		add_action('admin_footer', array( $this, 'switch_coupon_dropdowns' ), 99);
		add_action('woocommerce_check_cart_items', array( $this, 'spinwheel_check_cart_items' ));
		//Spin Wheel coupon tab @ WooCommerce my account
		add_action('init', array( $this, 'woo_wheel_user_coupon_end_point' ));
		add_filter('query_vars', array( $this, 'woo_wheel_user_coupon_query_vars' ), 0);
		add_filter('woocommerce_account_menu_items', array( $this, 'woo_wheel_account_menu_items' ));
		add_action('woocommerce_account_Spin-Wheel-Coupons_endpoint', array( $this, 'woo_wheel_account_coupon_page' ));

		add_action('woocommerce_removed_coupon', array( $this, 'woo_wheel_remove_coupon' ));
	}

	public function get_type_image() {

		$attachment_url = false;

		if ($this->attachment_id ) {
			$attachment_url = wp_get_attachment_url($this->attachment_id);
		}

		if ($attachment_url ) {
			return $attachment_url;
		}

		return false;
	}

	public function spinwheel_check_cart_items() {

		global $woocommerce;

		$order_total = WC()->cart->get_subtotal();
		$winning_coupon_code = get_option('woo_wheel_settings_winning_text_coupon');

		$generated_coupon_code = get_option('woo_wheel_get_coupon_code', true);
		$coupon_code = isset($_COOKIE['woo_wheel_gen_code']) ? sanitize_text_field(wp_unslash($_COOKIE['woo_wheel_gen_code'])) : '';

		$wheel_slices = woo_wheel_get_option('woo_wheel_slices');

		$latest_coupons = get_posts(
			array(
			'posts_per_page'   => 1,
			'orderby'          => 'date',
			'order'            => 'desc',
			'post_type'        => 'shop_coupon',
			'post_status'      => 'publish',
			)
		);

		$latest_coupon = !empty($latest_coupons) ? $latest_coupons[0] : null;
		$applied_coupons = WC()->cart->get_applied_coupons();
		$is_coupon_valid = null != $latest_coupon ? woo_wheel_is_coupon_valid($latest_coupon->post_name) : false;

		$add_coupon_to_cart_option = get_option('woo_wheel_settings_coupon_add_to_cart');
		
	  
		if (is_array($applied_coupons) && true == $is_coupon_valid &&  'yes' == $add_coupon_to_cart_option && !WC()->cart->is_empty() && !empty($coupon_code) && $latest_coupon && is_page('cart') && !empty($winning_coupon_code) && is_user_logged_in()) {

			// do not apply a coupon if already applied. 
			if (in_array(lcfirst($coupon_code), WC()->cart->get_applied_coupons()) ) {
				return;
			}

			echo '<script>alert("Coupon added to the cart")</script>'; 
			$user_id = get_current_user_id();
			$user_data = get_userdata($user_id);
			$cart_url = wc_get_cart_url();
			$checkout_url = wc_get_checkout_url();
			$site_url = home_url();

			$date = '';
			if (isset($_GET['date']) ) {
				$date = sanitize_text_field(wp_unslash($_GET['date']));
			}

			$date_query = woo_wheel_get_date_query($date);

			$paged = ( isset($_GET['paged']) ) ? intval($_GET['paged']) : 1;

			$args = array(
			'posts_per_page' => 10,
			'paged' => $paged,
			'post_type' => 'ww_stats',
			'post_status' => 'publish',
			'date_query' => $date_query,
			);
			$wheel_logs = new WP_Query($args);

			$prize      = get_post_meta($wheel_logs->post->ID, 'coupon_value', true);

			
			if (isset($_COOKIE['woo_wheel_gen_code']) && strtolower($coupon_code) == strtolower($generated_coupon_code) ) {
				$coupon_code = sanitize_text_field(wp_unslash($_COOKIE['woo_wheel_gen_code']));
				WC()->cart->add_discount( sanitize_text_field( lcfirst( $coupon_code ) ) );
			} 

		}
	}

	public function woo_wheel_remove_coupon( $coupon ) {
		setcookie('woo_wheel_gen_code', '', time() - 3600, '/');
	}

	public function woo_wheel_user_coupon_end_point() {
		add_rewrite_endpoint('Spin-Wheel-Coupons', EP_ROOT | EP_PAGES);
	}

	public function woo_wheel_user_coupon_query_vars( $vars ) {
		$vars[] = 'Spin-Wheel-Coupons';
		return $vars;
	}
	public function woo_wheel_account_menu_items( $items ) {
		$spin_wheel_my_account_tab = get_option('woo_wheel_settings_my_account_coupon_tab_title');
		/**
		* Filter to modify the my account tab.
		*
		* @since  1.0.0
		* @param  string $spin_wheel_my_account_tab the title name of my account tab
		* @return string Modified my accouny tab.
		*/
		$tab = apply_filters('change_spin_wheel_my_account_tab', $spin_wheel_my_account_tab);
		$items['Spin-Wheel-Coupons'] = $tab;
		return $items;
	}

	public function woo_wheel_account_coupon_page() {
		$user_id = get_current_user_id();
		$user_coupons = get_user_meta($user_id, 'whoo_wheel_user_coupons', true);

		if (!empty($user_coupons)) {
			echo '<table class="wp-list-table widefat fixed striped table-view-list posts">';
			echo '<thead>';
			echo '<tr>';
			echo '<th scope="col" id="coupon_code" class="manage-column column-coupon_code column-primary">Code</th>';
			echo '<th scope="col" id="amount" class="manage-column column-amount">Discount</th>';
			echo '<th scope="col" id="expiry_date" class="manage-column column-expiry_date">Expiry date</th>';
			echo '<th scope="col" id="usage" class="manage-column column-usage">Status</th>';
			echo '</tr>';
			echo '</thead>';
			echo '<tbody id="the-list">';

			foreach ( $user_coupons as $coupon ) {
				$wc_coupon = new WC_Coupon($coupon);
				$coupon_status = esc_html($wc_coupon->get_usage_count()) ==  esc_html($wc_coupon->get_usage_limit()) ? 'Used' :'Available';
				$discount      = esc_html($wc_coupon->get_discount_type()) == 'percent' ? esc_html($wc_coupon->get_amount()) . '%' : '$' . esc_html($wc_coupon->get_amount());
				if ($wc_coupon->get_id()) {
					echo '<tr>';
					echo '<td>' . esc_html($wc_coupon->get_code()) . '</td>';
					echo '<td>' . esc_html($discount) . '</td>';
					echo '<td>' . esc_html($wc_coupon->get_date_expires()->format('Y-m-d H:i:s')) . '</td>';
					printf('<td> <span class="woo_wheel_user_coupon_%s"> %s </span> </td>', esc_html(strtolower($coupon_status)), esc_html($coupon_status));
					echo '</tr>';
				}
			}

			echo '</tbody>';
			echo '</table>';
		}
	}

	public function frontend_scripts_styles() {

		$user_id = get_current_user_id();

		wp_register_script('woo-wheel-script', WOO_WHEEL_DIR . 'js/script.js', array( 'jquery' ), '1.4');

		$variables = array(
		'appearance_delay' => woo_wheel_get_option('woo_wheel_settings_appearance_delay'),
		'occurance_delay'  => woo_wheel_get_option('woo_wheel_settings_occurance_delay'),
		'scroll_trigger_percentage_desktop'  => woo_wheel_get_option('woo_wheel_settings_scroll_trigger_desktop'),
		'scroll_trigger_percentage_mobile'  => woo_wheel_get_option('woo_wheel_settings_scroll_trigger_mobile'),
		'idle_time'  => woo_wheel_get_option('woo_wheel_settings_scroll_trigger_inactive_sensor'),

		'hide_on_mob'      => woo_wheel_get_option('woo_wheel_settings_hide_on_mob'),
		'number_of_plays'  => woo_wheel_get_number_of_plays($user_id),
		'd_exit_intent'    => woo_wheel_get_option('woo_wheel_settings_section_d_intent'),
		'm_exit_intent'    => woo_wheel_get_option('woo_wheel_settings_section_m_intent'),
		'ajax_url'         => admin_url('admin-ajax.php'),
		'ajax_nonce'       => wp_create_nonce('save_log'),
		);
		wp_localize_script('woo-wheel-script', 'woo_wheel_display', $variables);

		wp_enqueue_script('woo-wheel-script');
	}

	public function admin_scripts_styles() {
		wp_enqueue_style('circle-progress-bar', WOO_WHEEL_DIR . 'css/circle.css', array(), '1.4');
		wp_enqueue_style('admin-style', WOO_WHEEL_DIR . 'css/admin-style.css', array(), '1.4');
		wp_enqueue_media();
		wp_register_script('wheel-slices-js', WOO_WHEEL_DIR . 'js/wheel-slices.js', array( 'jquery' ), '1.4');
		$variables = array(
		'ajax_url'           => admin_url('admin-ajax.php'),
		'ajax_nonce'         => wp_create_nonce('remove_image'),
		'uploadtitle'           => esc_js(esc_attr__('Spin Wheel Image', 'woo-wheel')),
		'uploadbutton'          => esc_js(esc_attr__('Use as Image', 'woo-wheel')),
		'fieldName'              => '[spin_wheel_woocommerce][fieldname]',
		'changeImage'        => esc_js(__('Change Image', 'woo-wheel')),
		);
		wp_localize_script('wheel-slices-js', 'woo_wheel_slices', $variables);
		wp_enqueue_script('jquery-ui-datepicker');
		wp_enqueue_script('wheel-slices-js');
	}

	public function add_settings_tab( $settings_tabs ) {
		$settings_tabs['woo_wheel_settings'] = __('Spin Wheel', 'woo-wheel');
		return $settings_tabs;
	}

	public function settings_tab() {

		$allowd_html = array(
		'ul' => array(
		'class' => 'subsubsub',
		),
		'li' => array(),
		'a'  => array(
		'href'  => array(),
		'class' => array(),
		),
		'br' => array(
		'class' => array(),
		),
		);

		echo wp_kses($this->woo_wheel_sections_html(), $allowd_html);

		woocommerce_admin_fields($this->get_settings());
	}

	public function update_settings() {
		woocommerce_update_options($this->get_settings());
	}

	public function get_settings() {

		$settings = array();

		if (! isset($_GET['section']) || empty($_GET['section']) ) {

			$settings = array(
			'general_title'           => array(
			'name'    => __('General Settings', 'woo-wheel'),
			'type'    => 'title',
			'default' => 'GET YOUR CHANCE TO WIN A PRIZE!',
			'desc'    => '',  //__('', 'woo-wheel'),
			'id'      => 'woo_wheel_general_title',
			),
			'enable_sound'          => array(
			'name'     => __('Enable Sound', 'woo-wheel'),
			'desc'     => __('Enable/Disable pin sound.', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_section_enable_sound',
			'default'  => 'no',
			'type'     => 'checkbox',
			'desc_tip' => true,
			),
			'enable_limited_time'   => array(
			'name'     => __('Display for limited time', 'woo-wheel'),
			'desc'     => __('Enable wheel for limited time. <strong>Current server time: ' . gmdate('d-m-Y h:i a') . '</strong>', 'woo-wheel'),
			'id'       => 'woo_wheel_enable_limited_time',
			'default'  => 'no', // WooCommerce >= 2.0
			'type'     => 'checkbox',
			'desc_tip' => true,
			),
			'wheel_start_on'        => array(
			'name' => __('Start Time', 'woo-wheel'),
			'type' => 'date_time',
			'css'  => 'width:150px;',
			'desc' => __('Set your time.', 'woo-wheel'),
			'id'   => 'wheel_start_date',
			),
			'wheel_end_on'          => array(
			'name' => __('End Time', 'woo-wheel'),
			'type' => 'date_time',
			'css'  => 'width:150px;',
			'desc' => __('Set your time.', 'woo-wheel'),
			'id'   => 'wheel_end_date',
			),
			'section_end8'          => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_coupon_section',
			),
			'popup_title_data'      => array(
			'name' => __('Wheel Slider', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_popup_title_data',
			),
			'popup_title'           => array(
			'name'    => __('Title', 'woo-wheel'),
			'type'    => 'text',
			'default' => 'GET YOUR CHANCE TO WIN A PRIZE!',
			'desc'    => __('Use <em></em> to emphasise text (it will have a different color).', 'woo-wheel'),
			'id'      => 'woo_wheel_popup_title',
			),
			'explainer_text'        => array(
			'name'     => __('Explainer Text', 'woo-wheel'),
			'type'     => 'textarea',
			'default'  => 'Click the Spin Wheel button and get a chance to win one of our amazing prizes!',
			'desc'     => __('A short paragraph explaining how it works.', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_explainer_text',
			'desc_tip' => true,
			),
			'disclaimer_text'       => array(
			'name'     => __('Disclaimer Text', 'woo-wheel'),
			'type'     => 'textarea',
			'default'  => 'Cheaters will not receive their prize.',
			'desc'     => __('Add a short paragraph explaining the rules & regulations.', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_disclaimer_text',
			'desc_tip' => true,
			),
			'spin_button'           => array(
			'name'    => __('Spin Button Text', 'woo-wheel'),
			'type'    => 'text',
			'default' => 'Spin Now',
			'desc'    => __('This text will appear on the button the visitor has to click to spin the wheel.', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_spin_button',
			),
			'winning_title'         => array(
			'name'    => __('Winning Title', 'woo-wheel'),
			'type'    => 'text',
			'default' => 'HURRAY! YOUVE WON A PRIZE. LUCKY YOU!',
			'desc'    => __('This title will appear after a player hits a winning segment. Use {s} to denote the segment`s label.', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_winning_title',
			),
			'winning_text_coupon'   => array(
			'name'    => __('Winning Text For Coupons', 'woo-wheel'),
			'type'    => 'text',
			'default' => 'Nicely done! You can use the coupon code below to get discount on items',
			'desc'    => __('This text will appear below the winning title and above coupon code after a player hits a winning coupon-segment.', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_winning_text_coupon',
			),
			'winning_text_link'     => array(
			'name'    => __('Winning Text For Links', 'woo-wheel'),
			'type'    => 'text',
			'default' => 'Nicely done! You can use the link below to get your book',
			'desc'    => __('This text will appear below the winning title after a player hits a winning link-segment.', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_winning_text_link',
			),
			'lossing_title'          => array(
			'name' => __('Losing Title', 'woo-wheel'),
			'type' => 'text',
			'desc' => __('This title will appear after a player hits a losing segment.', 'woo-wheel'),
			'id'   => 'woo_wheel_settings_losing_title',
			),
			'lossing_text'           => array(
			'name'    => __('Losing Text', 'woo-wheel'),
			'type'    => 'text',
			'default' => 'AAH! YOU HAVE LOST THE CHANCE',
			'desc'    => __('This text will appear below the losing title after a player hits a losing segment.', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_losing_text',
			),
			'done_text'             => array(
			'name'    => __('"Done" Button Text', 'woo-wheel'),
			'type'    => 'text',
			'default' => 'I`m done playing',
			'desc'    => __('When the player has done playing, this button will appear to allow to close the popup.', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_done_text',
			),
			'section_end2'          => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_popup_title_data',
			),
			'clickable_btn'         => array(
			'name' => __('Clickable Button', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_clickable_tap',
			),
			'enable_clickable_btn'  => array(
			'name'    => __('Enable Button', 'woo-wheel'),
			'desc'    => __('Button will appear on bottom.', 'woo-wheel'),
			'id'      => 'woo_wheel_enable_clickable_btn',
			'default' => 'no', 
			'type'    => 'checkbox',
			),
			'clickable_button_text' => array(
			'name'    => __('Clickable Button Text', 'woo-wheel'),
			'type'    => 'text',
			'default' => 'Try Lucky Wheel ✌️',
			'desc'    => __('Button text which will appear on bottom of the screen.', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_clickable_button_text',
			),
			'image_upload'          => array(
			'name' => __('Clickable Button Icon', 'woo-wheel'),
			'type' => 'image_upload',
			'desc' => __('Select the image for clickable button which appears right bottom of your page ( size should be 80x80 ).', 'woo-wheel'),
			'id'   => 'woo_wheel_clickable_tab_image',
			),
			'wheel_center_image'    => array(
			'name' => __('Wheel Center Logo', 'woo-wheel'),
			'type' => 'wheel_center_image',
			'desc' => __('Add an image which will appear in the center of wheel ( size should be 150x150 ).', 'woo-wheel'),
			'id'   => 'woo_wheel_wheel_center_logo',
			),
			'logo'                  => array(
			'name' => __('Logo', 'woo-wheel'),
			'type' => 'logo',
			'desc' => __('Add an image which will appear above the title ( max size should be 150x150 )', 'woo-wheel'),
			'id'   => 'woo_wheel_logo',
			),
			'section_end3'          => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_clickable_tap',
			), 'my_account_coupon_tab'      => array(
			'name' => __('My Account Coupon Tab', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_my_account_tab_title',
			),
			'my_account_coupon_tab_title'          => array(
			'name' => __('Tab Label', 'woo-wheel'),
			'type' => 'text',
			'default' => 'Spin Wheel Coupons',
			'desc' => __('Label of Tab in My Account', 'woo-wheel'),
			'id'   => 'woo_wheel_settings_my_account_coupon_tab_title',
			),
			'section_end4'          => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_my_account_tab_title',
			),
			'privacy_title_data'      => array(
			'name' => __('Privacy Policy', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_privacy_title_data',
			),
			'privacy_policy_enable'   => array(
			'name'     => __('Enable', 'woo-wheel'),
			'desc'     => '', //__('', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_section_privacy_policy_enable',
			'default'  => 'no', 
			'type'     => 'checkbox',
			'desc_tip' => true,
			),
			'privacy_title'          => array(
			'name' => __('Label', 'woo-wheel'),
			'type' => 'text',
			'default' => 'Privacy Policy',
			'desc' => '', //__('', 'woo-wheel'),
			'id'   => 'woo_wheel_settings_privacy_title',
			),
			'privacy_dropdown'          => array(
			'name' => __('Privacy Page', 'woo-wheel'),
			'type' => 'select',
			'desc' => '', //__('', 'woo-wheel'),
			'id'   => 'woo_wheel_settings_privacy_dropdown',
			'options' => $this->get_all_pages(),
			),
			'section_end5'          => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_privacy_tap',
			),
			);
							
			/**
			 * Action triggered when displaying the general settings tab in the Woo Wheel settings.
			 *
			 * @since 1.0.0
			 */
			do_action('woo_wheel_setting_tab_general');

		} elseif (isset($_GET['section']) && 'slices' == $_GET['section'] ) {

			$settings = array(
			'wheel_slices'             => array(
			'name' => __('Add Wheel Slices', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_wheel_slices',
			),
			'slices'                   => array(
			'name'     => __('Slices', 'woo-wheel'),
			'type'     => 'slices',
			'desc'     => __('Add Wheel Slices'),
			'desc_tip' => true,
			'id'       => 'woo_wheel_settings_slices',
			),
			'wheel_slices_section_end' => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_wheel_slices',
			),
			);
			/**
			 * Action hook for adding custom sections to the Slices tab in the Woo Wheel settings.
			 *
			 * @since 1.0.0
			 */
			do_action('woo_wheel_setting_tab_slices');

		} elseif (isset($_GET['section']) && 'design' == $_GET['section'] ) {

			$settings = array(
			'wheel_design'             => array(
			'name' => __('Wheel Design', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_wheel_design',
			),
			'background_color'         => array(
			'name'     => __('Background Color', 'woo-wheel'),
			'type'     => 'color',
			'css'      => 'width:150px;',
			'default'  => '#1e9094',
			'desc'     => __('You can override the theme`s default backgroud color for the slider', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_background_color',
			'desc_tip' => true,
			),
			'border_color'             => array(
			'name'     => __('Border Color', 'woo-wheel'),
			'type'     => 'color',
			'css'      => 'width:150px;',
			'default'  => '#D0BD0C',
			'desc'     => __('You can select the wheel border color', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_border_color',
			'desc_tip' => true,
			),
			'heading_color'            => array(
			'name'     => __('Heading Color', 'woo-wheel'),
			'type'     => 'color',
			'css'      => 'width:150px;',
			'default'  => '#FFF',
			'desc'     => __('You can override theme`s default heading color', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_heading_color',
			'desc_tip' => true,
			),
			'text_color'               => array(
			'name'     => __('Text Color', 'woo-wheel'),
			'type'     => 'color',
			'css'      => 'width:150px;',
			'default'  => '#FFF',
			'desc'     => __('You can override theme`s default text color', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_text_color',
			'desc_tip' => true,
			),
			'button_background'        => array(
			'name'     => __('Button Background', 'woo-wheel'),
			'type'     => 'color',
			'css'      => 'width:150px;',
			'default'  => '#056478',
			'desc'     => __('You can override theme`s default button background color', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_button_background',
			'desc_tip' => true,
			),
			'button_text_color'        => array(
			'name'     => __('Button Text Color', 'woo-wheel'),
			'type'     => 'color',
			'css'      => 'width:150px;',
			'default'  => '#FFF',
			'desc'     => __('You can override theme`s default button background color', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_button_text_color',
			'desc_tip' => true,
			),
			'winning_text_color'       => array(
			'name'     => __('Winning Text Color', 'woo-wheel'),
			'type'     => 'color',
			'css'      => 'width:150px;',
			'default'  => '#FFF',
			'desc'     => __('You can override winning text color', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_winning_text_color',
			'desc_tip' => true,
			),
			'winning_text_bgcolor'     => array(
			'name'     => __('Winning Background Color', 'woo-wheel'),
			'type'     => 'color',
			'css'      => 'width:150px;',
			'default'  => '#076669',
			'desc'     => __('You can override winning text background color', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_winning_text_bgcolor',
			'desc_tip' => true,
			),
			'wheel_design_section_end' => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_wheel_design',
			),
			);
			/**
			 * Fires before the design settings tab in the Woo Wheel plugin settings page.
			 *
			 * This hook allows developers to add custom content to the design settings tab by attaching their own functions
			 * to this action hook.
			 *
			 * @since 1.0.0
			 */
			do_action('woo_wheel_setting_tab_design');

		} elseif (isset($_GET['section']) && 'triggers' == $_GET['section'] ) {
			$settings = array(
			'spin_wheel_triggers'               => array(
			'name' => __('Triggers', 'woo-wheel'),
			'type' => 'title',
			'desc' => 'Triggers will run depending on display settings.',
			'id'   => 'woo_wheel_settings_section_spin_wheel_triggers',
			),
			'spin_wheel_triggers_desktop'            => array(
			'name'     => __('Enable Desktop Exit Intent', 'woo-wheel'),
			'type'     => 'checkbox',
			'desc'     => '', // __('', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_spin_wheel_triggers_desktop',
			'desc_tip' => true,
			),
			'spin_wheel_triggers_mobile' => array(
			'name'     => __('Enable Mobile Exit Intent', 'woo-wheel'),
			'type'     => 'checkbox',
			'desc'     => '', // __('', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_spin_wheel_triggers_mobile',
			'desc_tip' => true,
			),
			
			'spin_wheel_scroll_trigger_desktop'      => array(
			'name'     => __('Scroll Trigger For Desktop', 'woo-wheel'),
			'type'     => 'number',
			'default'  => '',
			'desc'     => __('Show Wheel Slider after X% of the page', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_scroll_trigger_desktop',
			'desc_tip' => false,
			'custom_attributes' => array(
			'min'  => 1,
			'step' => 1,
			),
			),

			'spin_wheel_scroll_trigger_mobile'      => array(
			'name'     => __('Scroll Trigger For Mobile', 'woo-wheel'),
			'type'     => 'number',
			'default'  => '',
			'desc'     => __('Show Wheel Slider after X% of the page', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_scroll_trigger_mobile',
			'desc_tip' => false,
			'custom_attributes' => array(
			'min'  => 1,
			'step' => 1,
			),
			),
			'appearance_delay'        => array(
			'name'    => __('Time Delay', 'woo-wheel'),
			'type'    => 'number',
			'default' => '2',
			'desc'    => __('In Seconds', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_appearance_delay',
			'custom_attributes' => array(
			'min'  => 1,
			'step' => 1,
			),
			),

			'spin_wheel_scroll_trigger_inactive_sensor'      => array(
			'name'     => __('Inactive Sensor', 'woo-wheel'),
			'type'     => 'number',
			'default'  => '',
			'desc'     => __('In Seconds', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_scroll_trigger_inactive_sensor',
			'desc_tip' => false,
			'custom_attributes' => array(
			'min'  => 1,
			'step' => 1,
			),
			),

			'spin_wheel_scroll_trigger_by_class'      => array(
			'name'     => __('Trigger By Class', 'woo-wheel'),
			'type'     => 'text',
			'default'  => 'spin_wheel',
			'desc'     => __('Paste this class on any button, link or HTML element to display Spin Wheel', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_scroll_trigger_by_class',
			'readonly'     => true,                    
			),
			'spin_wheel_section_end' => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_triggers',
			),            
			);
		} elseif (isset($_GET['section']) && 'display' == $_GET['section'] ) {
			$settings = array(
			'on_events'               => array(
			'name' => __('On Events', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_on_events',
			),
			'after_signup'            => array(
			'name'     => __('After Signup', 'woo-wheel'),
			'type'     => 'checkbox',
			'desc'     => __('Trigger once customer registers him self on your site', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_after_signup',
			'desc_tip' => true,
			),
			'after_add_to_cart_event' => array(
			'name'     => __('After Add To Cart Event', 'woo-wheel'),
			'type'     => 'checkbox',
			'desc'     => __('Once first product is added to cart after Occurrence delay duration, Works on first product', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_after_add_to_cart_event',
			'desc_tip' => true,
			),
			'thankyou_page'           => array(
			'name'     => __('Thank You Page (after every order complete)', 'woo-wheel'),
			'type'     => 'checkbox',
			'desc'     => __('When ever order completes', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_thankyou_page',
			'desc_tip' => true,
			),
			'on_pages'                => array(
			'name'     => __('Wheel on Pages', 'woo-wheel'),
			'type'     => 'radio',
			'class'    => 'woo_wheel_pages',
			'options'  => array(
						'all_pages'      => __('All Pages', 'woo-wheel'),
						'specific_pages' => __('Specific Pages', 'woo-wheel'),
			),
			'desc'     => __('Display on pages after Occurrence delay', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_on_pages',
			'desc_tip' => true,
			),
			'select_pages'            => array(
			'name'     => __('Select Pages', 'woo-wheel'),
			'type'     => 'multiselect',
			'class'    => 'wc-enhanced-select',
			'placeholder'    => __('Choose Pages', 'woo-wheel'),
			'options'  => $this->get_all_pages(),
			'desc'     => __('Use this option only if enable on specific pages', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_select_pages',
			'css'      => 'select-pages',
			'desc_tip' => true,
			),
			'on_posts'                => array(
			'name'     => __('Wheel on Post Types', 'woo-wheel'),
			'type'     => 'radio',
			'class'    => 'woo_wheel_posts',
			'options'  => array(
						'all_posts'      => __('All Posts', 'woo-wheel'),
						'specific_posts' => __('Specific Posts', 'woo-wheel'),
			),
			'desc'     => __('Display wheel on Post Types', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_on_posts',
			'desc_tip' => true,
			),
			'select_posts'            => array(
			'name'     => __('Select Posts', 'woo-wheel'),
			'type'     => 'multiselect',
			'class'    => 'wc-enhanced-select',
			'placeholder'    => __('Choose Post Types', 'woo-wheel'),
			'options'  => $this->get_all_posts(),
			'desc'     => __('Use this option only if enable on specific post types', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_select_posts',
			'css'      => 'select-pages',
			'desc_tip' => true,
			),
			'occurance_delay'         => array(
			'name'    => __('Occurrence Delay', 'woo-wheel'),
			'type'    => 'text',
			'default' => '1',
			'css'     => 'width:150px;',
			'desc'    => __('Show popup again after set no. of days', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_occurance_delay',
			),
			'on_events_section_end'   => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_on_events',
			),
			'snowfall'                => array(
			'name' => __('Snow fall', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_snowfall',
			),
			'enable_snowfall'         => array(
			'name'     => __('Enable Snowfall', 'woo-wheel'),
			'type'     => 'checkbox',
			'desc'     => __('Display on specific pages after Occurrence delay', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_enable_snowfall',
			'desc_tip' => true,
			),
			'snowfall_flakes_no'      => array(
			'name'     => __('Number of flakes', 'woo-wheel'),
			'type'     => 'number',
			'default'  => 50,
			'desc'     => __('Display on specific pages after Occurrence delay', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_snowfall_flakes_no',
			'desc_tip' => true,
			'custom_attributes' => array(
			'min'  => 1,
			'step' => 1,
			),
			),
			'snowfall_flakes_speed'   => array(
			'name'     => __('Snow Falling Speed', 'woo-wheel'),
			'type'     => 'number',
			'default'  => 40,
			'desc'     => __('Display on specific pages after Occurrence delay', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_snowfall_flakes_speed',
			'desc_tip' => true,
			'custom_attributes' => array(
			'min'  => 1,
			'step' => 1,
			),
			),
			'hide_on_mob'             => array(
			'name'     => __('Hide on Mobile', 'woo-wheel'),
			'type'     => 'checkbox',
			'desc'     => __('If checked, wheel popup will hide on mobile screen', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_hide_on_mob',
			'desc_tip' => true,
			),         
			'snowfall_section_end'    => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_snowfall',
			),
			'exit_intent_section_end' => array(
			'type' => 'sectionend',
			'id'   => 'woo_wheel_settings_section_exit_intent',
			),
			);
			/**
			 * Fires before the display settings tab of the Woo Wheel plugin.
			 *
			 * @since 1.0.0
			 */
			do_action('woo_wheel_setting_tab_display');
		} elseif (isset($_GET['section']) && 'coupons' == $_GET['section'] ) {
			$settings = array(
			'coupon_settings'       => array(
			'name'    => '', // __('', 'woo-wheel'),
			'type'    => 'title',
			'css'     => '',
			'default' => '',
			'desc'    => '', //__('', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_coupon',
			),
			'coupon_duration'       => array(
					 'name'    => __('Coupon Duration', 'woo-wheel'),
					 'type'    => 'number',
					 'css'     => 'width:150px;',
					 'default' => '1',
					 'desc'    => __('How long will before the coupon expires? Tip: You can inform user of this via the content settings of each wheel ', 'woo-wheel'),
					 'id'      => 'woo_wheel_settings_coupon_duration',
					 'custom_attributes' => array(
						'min'  => 1,
						'step' => 1,
			),
			),
			'coupon_duration_time'  => array(
					 'name'     => '',
					 'desc'     => __('How long will before the coupon expires? Tip: You can inform user of this via the content settings of each wheel.', 'woo-wheel'),
					 'id'       => 'woo_wheel_settings_coupon_duration_time',
					 'css'      => 'width:150px;',
					 'default'  => 'hour', // WooCommerce >= 2.0
					 'type'     => 'select',
					 'options'  => array(
						 'hour' => __('Hour(s)', 'woo-wheel'),
						 'day'  => __('Day(s)', 'woo-wheel'),
			),
					 'desc_tip' => true,
			),
			'coupon_add_to_cart' => array(
			'name'    => __('Auto add coupon to cart', 'woo-wheel'),
			'type'    => 'checkbox',
			'default' => 'no',
			'desc_tip' => true,
			'desc'    => __('Works when product is added to the cart', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_coupon_add_to_cart',
			),
				  'coupon_enable_cart_redirect' => array(
			'name'    => __('Enable Cart Redirect', 'woo-wheel'),
			'type'    => 'checkbox',
			'default' => 'no',
			'desc_tip' => true,
			'desc'    => __('Enable cart redirect after successfully added coupon to list', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_coupon_cart_redirect',
			),
			'coupon_section'        => array(
			'name' => __('Coupon Settings', 'woo-wheel'),
			'type' => 'title',
			'desc' => '',
			'id'   => 'woo_wheel_settings_section_coupon_section',
			),
			);
		} elseif (isset($_GET['section']) && 'email' == $_GET['section'] ) {
			$settings = array(
			'email_settings'       => array(
			'name'    => '', // __('', 'woo-wheel'),
			'type'    => 'title',
			'css'     => '',
			'default' => '',
			'desc'    => '', // __('', 'woo-wheel'),
			'id'      => 'woo_wheel_settings_email',
			),
			'email_enable'   => array(
			'name'     => __('Enable Email', 'woo-wheel'),
			'desc'     => '', //__('', 'woo-wheel'),
			'id'       => 'woo_wheel_settings_section_email_enable',
			'default'  => 'no',
			'type'     => 'checkbox',
			'desc_tip' => true,
			),
			'email_template_win_text'        => array(
			'name'     => __('Email Template - Win', 'woo-wheel'),
			'type'     => 'textarea',
			'class'     => 'win_guide_text',
			'css'      => 'min-width: 300px; min-height:150px',                
			'default'  => 'Hi ! You have won a free coupon, enjoy and keep shopping',
			'id'       => 'woo_wheel_settings_email_template_win_text',
			// translators: %1 is for username and %2 is for coupon code
			'desc_tip'        => __('Email message upon winning. Available tags %1$username% %2$coupon%', 'woo-wheel'),
			),
			'email_template_lose_text'        => array(
			'name'     => __('Email Template - Lose', 'woo-wheel'),
			'type'     => 'textarea',
			'css'      => 'min-width: 300px; min-height:150px',    
			'default'  => 'Hi ! Oohh! you missed the chance. Better luck next time.',
			'id'       => 'woo_wheel_settings_email_template_lose_text',
			// Translators: %username% is a placeholder that will be replaced with the user's name.
			'desc_tip' => __('Email message upon losing. Available tags %username%', 'woo-wheel'),
			),
			'email_section'        => array(
			'name' => __('Email Template', 'woo-wheel'),
			'type' => 'title',
			'desc' => 'Available tags: %username% %coupon% %site_url% %cart_url% % %checkout_url% %winning_text%',
			'id'   => 'woo_wheel_settings_section_email_section',
			),
			);
		} elseif (isset($_GET['section']) && 'stats' == $_GET['section'] ) {
			/**
			 * Fires before the Woo Wheel plugin settings tab "Stats" is rendered.
			 *
			 * @since 1.0.0
			 */
			do_action('woo_wheel_setting_tab_stats');
		} elseif (isset($_GET['section']) && 'preview' == $_GET['section'] ) {

			$html = '<a href="' . home_url() . '?woo_wheel" target="_blank" class="ww_preview">See Preview</a>';

			$allowd_html = array(
			'a' => array(
			'href'  => array(),
			'class' => 'ww_preview',
			'target' => '_blank',
			),
			);
			echo wp_kses($html, $allowd_html);
			/**
			 * Triggered before the display of the preview tab in the Woo Wheel settings page.
			 *
			 * @since 1.0.0
			 */
			do_action('woo_wheel_setting_tab_preview');
		}
		/**
		 * Filters the settings array for the Woo Wheel plugin.
		 *
		 * @param  array $settings The settings array.
		 * @return array The filtered settings array.
		 *
		 * @since 1.0.0
		 */
		return apply_filters('woo_wheel_settings_settings', $settings);
	}

	public function get_all_pages() {

		$wp_pages = array();
		$pages    = get_pages();
		foreach ( $pages as $page ) {
			$wp_pages[ $page->ID ] = $page->post_title;
		}
		/**
		 * Filters the wp pages array for the Woo Wheel plugin.
		 *
		 * @param  array $wp_pages The pages array.
		 * @return array The filtered pages array.
		 *
		 * @since 1.0.0
		 */
		return apply_filters('woo_wheel_display_on_pages', $wp_pages);
	}

	public function get_all_posts() {
		$posts = array();
		$args = array(
		'post_type'      => 'post',
		'orderby'        => 'ID',
		'post_status'    => 'publish',
		'order'          => 'DESC',
		'posts_per_page' => -1,
		);
		$result = new WP_Query($args);
		if ($result->have_posts()) {
			while ($result->have_posts()) {
				$result->the_post();
				$posts[get_the_title()] = get_the_title();
			}
			wp_reset_postdata();
		}
		/**
		 * Filters the wp posts array for the Woo Wheel plugin.
		 *
		 * @param  array $posts The posts array.
		 * @return array The filtered posts array.
		 *
		 * @since 1.0.0
		 */
		return apply_filters('woo_wheel_display_on_posts', $posts);
	}

	public function get_woo_wheel_sections() {
		$sections = array(
		''         => __('General', 'woo-wheel'),
		'design'   => __('Design', 'woo-wheel'),
		'slices'   => __('Slices', 'woo-wheel'),
		'display' => __('Display', 'woo-wheel'),
		'coupons'  => __('Coupon', 'woo-wheel'),
		'email'    => __('Email Template', 'woo-wheel'),
		'triggers'  => __('Triggers', 'woo-wheel'),
		'stats'    => __('Stats', 'woo-wheel'),
		'preview'  => __('Preview', 'woo-wheel'),
		);
		/**
		 * Filters the sections for the Woo Wheel settings page.
		 *
		 * @param  array $sections An array of section names.
		 * @return array The filtered array of section names.
		 *
		 * @since 1.0.0
		 */
		return apply_filters('woo_wheel_get_sections', $sections);
	}

	public function woo_wheel_sections_html() {
		global $current_section;

		$sections = $this->get_woo_wheel_sections();
		if (empty($sections) || 1 === count($sections) ) {
			return;
		}
		$html       = '<ul class="subsubsub">';
		$array_keys = array_keys($sections);
		foreach ( $sections as $id => $label ) {
			$html .= '<li><a href="' . admin_url('admin.php?page=wc-settings&tab=woo_wheel_settings&section=' . sanitize_title($id)) . '" class="' . ( $current_section == $id ? 'current' : '' ) . '">' . $label . '</a> ' . ( end($array_keys) == $id ? '' : '|' ) . ' </li>';
		}
		$html .= '</ul><br class="clear" />';

		return $html;
	}

	public function add_wheel_field_slices( $value ) {
		$option_value = (array) WC_Admin_Settings::get_option($value['id']);
		$description  = WC_Admin_Settings::get_field_description($value);
		$slices = woo_wheel_get_option('woo_wheel_slices');
		?>
		<tr valign="top">
			<td class="forminp" id="bacs_accounts">
				<div class="wc_input_table_wrapper">
					<table id='spin_wheel_woocommerce_tbl' class="widefat wc_input_table sortable" cellspacing="0">
						<thead>
							<tr>
								<th class="checkbox">&nbsp;</th>
								<th class="sort">&nbsp;</th>                                
								<th><?php esc_html_e('Type', 'woocommerce'); ?></th>
								<th><?php esc_html_e('Value', 'woocommerce'); ?></th>
								<th><?php esc_html_e('Text', 'woocommerce'); ?></th>
								<th><?php esc_html_e('Chance', 'woocommerce'); ?></th>
								<th><?php esc_html_e('Color', 'woocommerce'); ?></th>
							</tr>
						</thead>
						<tbody class="accounts">
		<?php
		$allowd_html = array(
		'tr'     => array(
								'class' => 'woo-wheel-slices',
		),
		'li'     => array(),
		'td'     => array(
		   'class' => array(),
		),
		'span'   => array(
		   'class' => array(),
		   'style' => array(),
		),
		'select' => array(
		   'id'    => array(),
		   'class' => array(),
		   'name'  => array(),
		   'style' => array(),
		),
		'option' => array(
		   'value'    => array(),
		   'selected' => array(),
		),
		'input'  => array(
		   'class'       => array(),
		   'name'        => array(),
		   'value'       => array(),
		   'style'       => array(),
		   'id'          => array(),
		   'type'        => array(),
		   'dir'         => array(),
		   'placeholder' => array(),
		),
		);

		echo wp_kses($this->get_saved_slices(), $allowd_html);
		?>
						</tbody>
						<tfoot>
							<tr>
								<th colspan="7"><a href="#" id="spin_wheel_woocommerce_btnAdd" class="add button"><?php esc_html_e('+ Add Slice', 'woocommerce'); ?></a> <a href="#" id="deleteButton" class="remove_rows button"><?php esc_html_e('Remove selected Slice(s)', 'woocommerce'); ?></a></th>
							</tr>
						</tfoot>
					</table>
				</div>
				<script type="text/javascript">
					jQuery(function() {
						jQuery('#bacs_accounts').on( 'click', 'a.add', function(){
							var size = jQuery('#bacs_accounts').find('tbody .account').length;
		<?php
		$allowd_html = array(
		'select' => array(
		'id'    => array(),
		'class' => array(),
		'name'  => array(),
		'style' => array(),
		),
		'option' => array(
									'value'    => array(),
									'selected' => array(),
		),
		);
		?>
							jQuery('<tr class="account">\
								<td class="checkbox_checked"><input type="checkbox" value="" name="woo_wheel_slices[checkbox][]" id="spinwheelcheckbox" class="checkbox_value"></td>\
									<td class="sort"></td>\
									<td><?php echo wp_kses($this->getCouponTypes(), $allowd_html); ?></td>\
									<td><?php echo wp_kses($this->getAvailableCouponsHtml('', 'new'), $allowd_html); ?><input type="text" name="woo_wheel_slices[value][]" class="coupon_text" placeholder="Slice Value" style="display:none"></td>\
									<td><input type="text" name="woo_wheel_slices[text][]" class="coupon_value" placeholder="Slice Text" /></td>\
									<td><input type="number" min="0" max="100" placeholder="in percent" name="woo_wheel_slices[chance][]" /></td>\
									<td><span class="colorpickpreview" style="background: #d55252">&nbsp;</span><input name="woo_wheel_slices[bg_color][]" id="bg_color" type="text" dir="ltr" style="width:150px !important;" value="#d55252" class="colorpick" placeholder=""></td>\
								</tr>').appendTo('#bacs_accounts table tbody');
								colorPicker();
								couponTypeChanged();
								//jQuery( '.ww_coupon_type' ).trigger('change');
								change_coupon_type();
							return false;
						});
						colorPicker();                        
					});
				</script>
			</td>
		</tr>
		<?php
	}

	public function woo_get_image_id() {
		$image_id = get_option('woo_wheel_clickable_tab_image');

		if (empty($image_id) ) {
			return true;
		}
		
		return $image_id;
	}

	public function add_wheel_image_upload( $value ) {
		wp_nonce_field('woo_wheel_icon_nonce', 'woo_wheel_icon_nonce_field');
		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<label for="<?php echo esc_attr($value['id']); ?>"><?php echo esc_html($value['title']); ?></label>
			</th>
			<td class="forminp forminp-text clickable-button-icon">
				<input name="clickable-tab" id="<?php echo esc_attr($value['id']); ?>" multiple="false" type="file" style="" value="" class="woo-wheel-logo" placeholder="">
		<?php
		$tab_image = get_option('woo_wheel_clickable_tab_image');

		if (false !== $tab_image && ! empty($tab_image) ) {


			$attachment_id = $this->woo_get_image_id();
			$image_url        = wp_get_attachment_url($attachment_id);

			$input_name_value = 'spin_woocommerce[attachment_id]';
			echo '<span class="dashicons dashicons-no remove-image" data-img="' . esc_attr($value['id']) . '"></span>
				<div class="ww-image-box">
				<img src="' . esc_url(wp_get_attachment_url($attachment_id)) . '" />
				<input type="hidden" value="' . esc_attr($attachment_id) . '" name="' . esc_attr($input_name_value) . '" />
				</div>';
			update_option('woo_wheel_clickable_tab_image', $tab_image);
		} else {


			echo '<div class="ww-image-box"><span>80x80</span></div>';
		}
		?>
			<span class="description"><?php echo esc_html($value['desc']); ?></span>
			</td>
		</tr>
		<?php
	}

	public function add_wheel_field_date_time( $value ) {

		$wheel_date = woo_wheel_get_option('woo_wheel_limited_date');
		wp_nonce_field('wheel_limited_time', 'wheel_limited_time_nonce');
		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<label for="<?php echo esc_attr($value['id']); ?>"><?php echo esc_html($value['title']); ?></label>
			</th>
			<td class="forminp forminp-text">
				<input name="<?php echo esc_attr($value['id']); ?>" id="<?php echo esc_attr($value['id']); ?>" style="<?php echo esc_attr($value['css']); ?>" type="text" style="" value="<?php echo ( isset($wheel_date[ $value['id'] ]['date']) ? esc_html($wheel_date[ $value['id'] ]['date']) : '' ); ?>" class="my-class" placeholder="Select Date">

				<select name="hour_<?php echo esc_attr($value['id']); ?>" style="width:80px">
		<?php
		for ( $i = 1; $i <= 12; $i++ ) {
			if (esc_html($wheel_date[ $value['id'] ]['hour']) == $i ) {
				echo '<option selected="selected" value="' . esc_html($wheel_date[ $value['id'] ]['hour']) . '">' . esc_html($wheel_date[ $value['id'] ]['hour']) . '</option>';
			} else {
				echo '<option value="' . esc_attr($i) . '">' . esc_attr($i) . '</option>';
			}
		}
		?>
				</select> <strong>:</strong>

				<select name="minute_<?php echo esc_attr($value['id']); ?>" style="width:80px">
		<?php
		for ( $i = 1; $i <= 59; $i++ ) {
			if (esc_html($wheel_date[ $value['id'] ]['minute']) == $i ) {
				echo '<option selected="selected" value="' . esc_html($wheel_date[ $value['id'] ]['minute']) . '">' . esc_html($wheel_date[ $value['id'] ]['minute']) . '</option>';
			} else {
				echo '<option value="' . esc_attr($i) . '">' . esc_attr($i) . '</option>';
			}
		}
		?>
				</select>
				<select name="time-format_<?php echo esc_attr($value['id']); ?>" style="width:80px;line-height: 1;"><option <?php echo ( isset($wheel_date[ $value['id'] ]['time_format']) && 'AM' == $wheel_date[ $value['id'] ]['time_format'] ? 'selected' : '' ); ?>>AM</option><option <?php echo ( isset($wheel_date[ $value['id'] ]['time_format']) && 'PM' == $wheel_date[ $value['id'] ]['time_format'] ? 'selected' : '' ); ?>>PM</option></select>
			<p class="description"><?php echo esc_attr($value['desc']); ?></p>
			</td>
		</tr>
		<?php
	}

	public function add_wheel_logo( $value ) {

		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<label for="<?php echo esc_attr($value['id']); ?>"><?php echo esc_html($value['title']); ?></label>
			</th>
			<td class="forminp forminp-text above-title">
				<input name="wheel-logo" id="<?php echo esc_attr($value['id']); ?>" multiple="false" type="file" style="" value="" class="ww-img-box" placeholder="">
		<?php
		$wheel_logo = get_option('woo_wheel_logo');
		if (false !== $wheel_logo && ! empty($wheel_logo) ) {
			echo '<span class="dashicons dashicons-no remove-image" data-img="' . esc_attr($value['id']) . '"></span><div class="woo-wheel-logo"><img src="' . esc_url($wheel_logo) . '" /></div>';
			update_option('woo_wheel_logo', $wheel_logo);
		} else {
			echo '<div class="woo-wheel-logo"><span>150x150</span></div>';
		}
		?>
			<span class="description"><?php echo esc_html($value['desc']); ?></span>
			</td>
		</tr>
		<?php
	}

	public function add_wheel_center_image( $value ) {
		wp_nonce_field('wheel_image', 'wheel_image_nonce');
		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<label for="<?php echo esc_attr($value['id']); ?>"><?php echo esc_html($value['title']); ?></label>
			</th>
			<td class="forminp forminp-text center-logo">
				<input name="wheel-center-logo" id="<?php echo esc_attr($value['id']); ?>" multiple="false" type="file" style="" value="" class="ww-wheel-logo" placeholder="">
		<?php
		$wheel_logo = get_option('woo_wheel_wheel_center_logo');

		if (false !== $wheel_logo && ! empty($wheel_logo) ) {
			echo '<span class="dashicons dashicons-no remove-image" data-img="' . esc_attr($value['id']) . '"></span><div class="ww-wheel-logoo"><img src="' . esc_url($wheel_logo) . '" /></div>';
			update_option('woo_wheel_wheel_center_logo', $wheel_logo);
		} else {
			echo '<div class="ww-wheel-logoo"><span>150x150</span></div>';
		}
		?>
			<span class="description"><?php echo esc_html($value['desc']); ?></span>
			</td>
		</tr>
		<?php
	}

	public function couponValue() {
		$html = '<input type="text" name="woo_wheel_slices[value][]" placeholder="20%" />';
		$generateCoupon = get_option('woo_wheel_settings_section_enable_sound');
		if (! empty($generateCoupon) && 'no' != $generateCoupon ) {
			$html = '<input type="text" readonly name="woo_wheel_slices[value][]" placeholder="Coupon Will Generate Automatically" />';
		}
		return $html;
	}

	public function getCouponTypes( $default = '' ) {

		$couponTypes = array(
		'coupon'      => 'Coupon',
		'coupon_auto' => 'Coupon Auto Generated',
		'no_prize'    => 'No Prize',
		'link'        => 'Link',
		);
		/**
		 * Filters the available coupon types for the Woo Wheel plugin.
		 *
		 * @param  array $couponTypes An array of available coupon types.
		 * @return array The modified array of coupon types.
		 *
		 * @since 1.0.0
		 */
		$couponTypes = apply_filters('woo_wheel_coupon_types', $couponTypes);
		$html        = '<select name="woo_wheel_slices[coupon_type][]" class="ww_coupon_type">';
		foreach ( $couponTypes as $couponKey => $couponType ) {
			$selected = ( $couponKey == $default ) ? 'selected' : '';
			$html    .= '<option value="' . $couponKey . '" ' . $selected . ' >' . $couponType . '</option>';
		}
		$html .= '</select>';
		return $html;
	}

	public function get_saved_slices() {
		$slices = get_option('woo_wheel_slices');
		$html = wp_nonce_field('wheel_slices', 'wheel_slices_nonce');
		?>
		
		<?php
		if (! empty($slices) && is_array($slices['woo_wheel_slices']['coupon_type']) ) {
			$total_slices = count($slices['woo_wheel_slices']['coupon_type']);
			$checkbox_value = '';
			$allowd_html = array(
			'select' => array(
			'id'    => array(),
			'class' => array(),
			'name'  => array(),
			'style' => array(),
			),
			'option' => array(
			'value'    => array(),
			'selected' => array(),
			),
			);
			for ( $i = 0; $i < $total_slices; $i++ ) {
				$style = '';
				if ('coupon' == $slices['woo_wheel_slices']['coupon_type'][ $i ] || 'coupon_auto' == $slices['woo_wheel_slices']['coupon_type'][ $i ] ) {
					$style = 'ww-hide';
				}
				if (isset($_REQUEST['wheel_slices_nonce']) || ( isset($_REQUEST['wheel_slices_nonce']) &&  wp_verify_nonce(sanitize_text_field(wp_unslash($_REQUEST['wheel_slices_nonce'])), 'wheel_slices') )) {
					$checkbox_value = isset($_POST['woo_wheel_slices']['checkbox'][$i]) ? sanitize_text_field(wp_unslash($_POST['woo_wheel_slices']['checkbox'][$i])) : '';
				}

				$html .= '<tr class="woo-wheel-slices">
				    <td class="checkbox_checked"><input type="checkbox" value="' . esc_attr($checkbox_value) . '" name="woo_wheel_slices[checkbox][]" id="spinwheelcheckbox" class="checkbox_value" data-row="' . esc_attr($total_slices) . '" ></td>
					<td class="sort"></td>
						
					<td>' . $this->getCouponTypes($slices['woo_wheel_slices']['coupon_type'][ $i ]) . '</td>

					<td><input type="text" value="' . esc_attr($slices['woo_wheel_slices']['value'][ $i ]) . '" class="coupon_text ' . $style . '" name="woo_wheel_slices[value][]" />' . wp_kses($this->getAvailableCouponsHtml(( isset($slices['woo_wheel_slices']['woo_coupon'][ $i ]) ) ? $slices['woo_wheel_slices']['woo_coupon'][ $i ] : 'A', $slices['woo_wheel_slices']['coupon_type'][ $i ]), $allowd_html) . '</td>
					<td><input type="text" value="' . esc_attr($slices['woo_wheel_slices']['text'][ $i ]) . '" name="woo_wheel_slices[text][]" class="coupon_value" /></td>
					<td><input type="text" value="' . esc_attr($slices['woo_wheel_slices']['chance'][ $i ]) . '" name="woo_wheel_slices[chance][]" /></td>
					<td><span class="colorpickpreview" style="background: ' . esc_attr($slices['woo_wheel_slices']['bg_color'][ $i ]) . '">&nbsp;</span><input type="text" value="' . esc_attr($slices['woo_wheel_slices']['bg_color'][ $i ]) . '" class="colorpick" style="width:150px !important" name="woo_wheel_slices[bg_color][]" /></td>                    
				</tr>';
			}
		}
		return $html;
	}

	public function save_slices() {

		if (! isset($_REQUEST['wheel_slices_nonce']) || ! wp_verify_nonce(sanitize_text_field(wp_unslash($_REQUEST['wheel_slices_nonce'])), 'wheel_slices') ) {
			return;
		}
		if (isset($_POST['woo_wheel_slices']) ) {

			if (! isset($_POST['woo_wheel_slices']['coupon_type']) ) {
				return;
			}

			$total_slices = count($_POST['woo_wheel_slices']['coupon_type']);
			$wheel_slice = array();
			$simple_coupons = array();
			for ( $i = 0; $i < $total_slices; $i++ ) {
				if (isset($_POST['woo_wheel_slices']['coupon_type'][ $i ]) 
					|| isset($_POST['woo_wheel_slices']['value'][ $i ]) 
					|| isset($_POST['woo_wheel_slices']['woo_coupon'][ $i ]) 
					|| isset($_POST['woo_wheel_slices']['text'][ $i ]) 
					|| isset($_POST['woo_wheel_slices']['chance'][ $i ]) 
					|| isset($_POST['woo_wheel_slices']['bg_color'][ $i ]) 
				) {
					$wheel_slice['coupon_type'][ $i ] = sanitize_text_field(wp_unslash($_POST['woo_wheel_slices']['coupon_type'][ $i ]));
					$wheel_slice['value'][ $i ]       = sanitize_text_field(wp_unslash($_POST['woo_wheel_slices']['value'][ $i ]));
					$wheel_slice['woo_coupon'][ $i ]  = sanitize_text_field(wp_unslash($_POST['woo_wheel_slices']['woo_coupon'][ $i ]));
					$wheel_slice['text'][ $i ]        = sanitize_text_field(wp_unslash($_POST['woo_wheel_slices']['text'][ $i ]));
					$wheel_slice['chance'][ $i ]      = intval($_POST['woo_wheel_slices']['chance'][ $i ]);
					$wheel_slice['bg_color'][ $i ]    = sanitize_text_field(wp_unslash($_POST['woo_wheel_slices']['bg_color'][ $i ]));

					$coupn_type = sanitize_text_field(wp_unslash($_POST['woo_wheel_slices']['coupon_type'][ $i ]));
					if ('coupon' ==  $coupn_type  ) {
						$coupon_id = absint(wp_unslash($_POST['woo_wheel_slices']['woo_coupon'][ $i ]));
						$coupon = new WC_Coupon($coupon_id);
						$coupon_code = $coupon->get_code();

						$simple_coupons[] = $coupon_code;
					}
				}
			}
			$slices['woo_wheel_slices'] = $wheel_slice;
			update_option('woo_wheel_slices', $slices);
			update_option('woo_manual_coupons', $simple_coupons);
		}
	}

	public function save_limited_time_wheel() {

		if (! isset($_REQUEST['wheel_limited_time_nonce']) || ! wp_verify_nonce(sanitize_text_field(wp_unslash($_REQUEST['wheel_limited_time_nonce'])), 'wheel_limited_time') ) {
			return;
		}
		$start_date         = array();
		$end_date           = array();
		$wheel_limited_date = array();
		if (isset($_POST['wheel_start_date']) 
			&& isset($_POST['hour_wheel_start_date']) 
			&& isset($_POST['minute_wheel_start_date']) 
			&& isset($_POST['time-format_wheel_start_date']) 
		) {
			$start_date['date']                     = sanitize_text_field(wp_unslash($_POST['wheel_start_date']));
			$start_date['hour']                     = sanitize_text_field(wp_unslash($_POST['hour_wheel_start_date']));
			$start_date['minute']                   = sanitize_text_field(wp_unslash($_POST['minute_wheel_start_date']));
			$start_date['time_format']              = sanitize_text_field(wp_unslash($_POST['time-format_wheel_start_date']));
			$wheel_limited_date['wheel_start_date'] = $start_date;
		}

		if (isset($_POST['wheel_end_date']) 
			&& isset($_POST['hour_wheel_end_date']) 
			&& isset($_POST['minute_wheel_end_date']) 
			&& isset($_POST['time-format_wheel_end_date']) 
		) {
			$end_date['date']        = sanitize_text_field(wp_unslash($_POST['wheel_end_date']));
			$end_date['hour']        = sanitize_text_field(wp_unslash($_POST['hour_wheel_end_date']));
			$end_date['minute']      = sanitize_text_field(wp_unslash($_POST['minute_wheel_end_date']));
			$end_date['time_format'] = sanitize_text_field(wp_unslash($_POST['time-format_wheel_end_date']));
			$wheel_limited_date['wheel_end_date'] = $end_date;
		}
		update_option('woo_wheel_limited_date', $wheel_limited_date);
	}

	public function get_default_image_id() {

		$image_id = get_option('default_image');

		if (empty($image_id) ) {
			return true;
		}
		
		return $image_id;
	}


	public function save_image( $post ) {
   
		$new_data = array();
		if (isset($_REQUEST['woo_wheel_icon_nonce_field']) &&  wp_verify_nonce(sanitize_text_field(wp_unslash($_REQUEST['woo_wheel_icon_nonce_field'])), 'woo_wheel_icon_nonce') ) {
			// Check if the 'spin_wheel_image' field is set in the $_POST array
			if (isset($_POST['spin_wheel_image'])) {
				// Sanitize and validate the attachment ID
				$attachment_id = absint($_POST['spin_wheel_image']);

				// Update the option only if the attachment ID is greater than 0
				if ($attachment_id > 0) {
					$new_data['attachment_id'] = $attachment_id;
					update_option('woo_wheel_clickable_tab_image', $new_data['attachment_id']);
				} 
			}
		}
	}

	public function remove_image() {

		check_ajax_referer('remove_image', 'security');

		$this->data = $_REQUEST;

		if (isset($this->data['image_id']) ) {
			$image_id = sanitize_text_field($this->data['image_id']);
			update_option($image_id, '');
		}

		wp_die();
	}

	public function getAvailableCouponsHtml( $default = '', $couponType = '' ) {

		$coupons = woo_wheel_getAvailableCoupons();

		$style_fix  = 'style="display:none"';
		$style_auto = 'style="display:none"';

		$html = '';
		if ('coupon_auto' == $couponType ) {
			$style_auto = 'style="display:block"';

			$html .= '<select name="woo_wheel_slices[woo_coupon][]" class="woo_coupon auto" ' . $style_auto . '>';
			$html .= '<option value="">-- Select Coupon --</option>';
			for ( $i = 1; $i <= 100; $i++ ) {
				$selected = ( $i == $default ) ? 'selected' : '';
				$html    .= '<option value="' . $i . '" ' . $selected . ' >' . $i . '%</option>';
			}
			$html .= '</select>';

		} elseif ('coupon' == $couponType ) {
			$style_fix = 'style="display:block"';

			$html .= '<select name="woo_wheel_slices[woo_coupon][]" class="woo_coupon fix" ' . $style_fix . '>';
			$html .= '<option value="">-- Select Coupon --</option>';
			foreach ( $coupons as $coupon ) {
				$selected = ( $coupon->ID == $default ) ? 'selected' : '';
				$html    .= '<option value="' . $coupon->ID . '" ' . $selected . ' >' . $coupon->post_title . '</option>';
			}

			$html .= '</select>';

		} elseif ('new' == $couponType ) {

			$style_fix = 'style="display:block"';

			$html .= '<select name="woo_wheel_slices[woo_coupon][]" class="woo_coupon fix" ' . $style_fix . '>';
			$html .= '<option value="">-- Select Coupon --</option>';
			foreach ( $coupons as $coupon ) {
				$selected = ( $coupon->ID == $default ) ? 'selected' : '';
				$html    .= '<option value="' . $coupon->ID . '" ' . $selected . ' >' . $coupon->post_title . '</option>';
			}

			$html .= '</select>';
		} else {

			$style_fix  = 'style="display:none"';
			$style_auto = 'style="display:none"';

			$html .= '<select name="woo_wheel_slices[woo_coupon][]" class="woo_coupon fix ww-hide">';
			$html .= '<option value="">-- Select Coupon h --</option>';
			foreach ( $coupons as $coupon ) {
				$selected = ( $coupon->ID == $default ) ? 'selected' : '';
				$html    .= '<option value="' . $coupon->ID . '" ' . $selected . ' >' . $coupon->post_title . '</option>';
			}

			$html .= '</select>';
		}
		return $html;
	}

	public function get_wc_coupons_dropdown() {

		$coupons = woo_wheel_getAvailableCoupons();

		$html  = '<select name="woo_wheel_slices[woo_coupon][]" class="woo_coupon fix">';
		$html .= '<option value="">-- Select Coupon --</option>';
		foreach ( $coupons as $coupon ) {
			$html .= '<option value="' . $coupon->ID . '">' . $coupon->post_title . '</option>';
		}

		$html .= '</select>';

		$html .= '<input type="text" name="woo_wheel_slices[value][]" class="coupon_text ww-hide" placeholder="Slice Value">';

		return $html;
	}

	public function get_wc_auto_coupons_dropdown() {
		$html  = '<select name="woo_wheel_slices[woo_coupon][]" class="woo_coupon auto">';
		$html .= '<option value="">-- Select Coupon --</option>';
		for ( $i = 1; $i <= 100; $i++ ) {
			$html .= '<option value="' . $i . '">' . $i . '%</option>';
		}
		$html .= '</select>';

		$html .= '<input type="text" name="woo_wheel_slices[value][]" class="coupon_text ww-hide" placeholder="Slice Value">';

		return $html;
	}

	public function other_fields() {

		$html  = '<input type="text" name="woo_wheel_slices[value][]" class="coupon_text" placeholder="Slice Value">';
		$html .= '<select name="woo_wheel_slices[woo_coupon][]" class="woo_coupon auto ww-hide">';
		$html .= '<option value="">-- Select Coupon --</option>';
		for ( $i = 1; $i <= 100; $i++ ) {
			$html .= '<option value="' . $i . '">' . $i . '%</option>';
		}
		$html .= '</select>';

		return $html;
	}

	public function switch_coupon_dropdowns() {

		$allowd_html = array(
		'select' => array(
		'id'    => array(),
		'class' => array(),
		'name'  => array(),
		'style' => array(),
		),
		'option' => array(
		'value'    => array(),
		'selected' => array(),
		),
		'input'  => array(
		'class'       => array(),
		'id'          => array(),
		'style'       => array(),
		'type'        => array(),
		'name'        => array(),
		'placeholder' => array(),
		),
		);
		?>
		<script>
		jQuery(document).ready(function() {
			change_coupon_type();
		});
		
		function change_coupon_type() {
			jQuery('.ww_coupon_type').change(function() {
				if ( jQuery(this).val() == 'coupon' ) {
					jQuery(this).closest('tr').find('td:nth-child(4)').html( '<?php echo wp_kses($this->get_wc_coupons_dropdown(), $allowd_html); ?>' );
				} else if ( jQuery(this).val() == 'coupon_auto' ) {
					jQuery(this).closest('tr').find('td:nth-child(4)').html( '<?php echo wp_kses($this->get_wc_auto_coupons_dropdown(), $allowd_html); ?>' );
				} else {
					jQuery(this).closest('tr').find('td:nth-child(4)').html( '<?php echo wp_kses($this->other_fields(), $allowd_html); ?>' );
				}
			});
		}
		</script>
		<?php
	}

	public function setting_tab_preview() {
		?>
		<style>
			button.woocommerce-save-button {
				display:none !important;
			}
		</style>
		<?php
	}
}
$Woo_Wheel_Settings_Tab = new Woo_Wheel_Settings_Tab();
