<?php
if (! defined('ABSPATH') ) {
	exit; // Exit if accessed directly
}

/**
 * Woo_Wheel_Stats class to handle Wheel stats
 */

class Woo_Wheel_Stats {



	public function __construct() {
		add_action('wp_ajax_save_log', array( $this, 'save_log' ));
		add_action('wp_ajax_nopriv_save_log', array( $this, 'save_log' ));
		add_action('init', array( $this, 'register_woo_wheel_stats' ));
		add_action('woo_wheel_setting_tab_stats', array( $this, 'stats_table' ));
	}

	public function before_save() {

		if (false == is_user_allowed_to_play() ) {
			echo 'Request Blocked!';
			die();
		}
	}

	public function register_woo_wheel_stats() {
		$labels = array(
		'name'               => _x('Stats', 'post type general name', 'woo-wheel'),
		'singular_name'      => _x('Stats', 'post type singular name', 'woo-wheel'),
		'menu_name'          => _x('Stats', 'admin menu', 'woo-wheel'),
		'name_admin_bar'     => _x('Stats', 'add new on admin bar', 'woo-wheel'),
		'add_new'            => _x('Add New', 'stats', 'woo-wheel'),
		'add_new_item'       => __('Add New Stats', 'woo-wheel'),
		'new_item'           => __('New Stats', 'woo-wheel'),
		'edit_item'          => __('Edit Stats', 'woo-wheel'),
		'view_item'          => __('View Stats', 'woo-wheel'),
		'all_items'          => __('All Stats', 'woo-wheel'),
		'search_items'       => __('Search Stats', 'woo-wheel'),
		'parent_item_colon'  => __('Parent Stats:', 'woo-wheel'),
		'not_found'          => __('No stats found.', 'woo-wheel'),
		'not_found_in_trash' => __('No stats found in Trash.', 'woo-wheel'),
		);

		$args = array(
		'labels'             => $labels,
		'description'        => __('Description.', 'woo-wheel'),
		'public'             => false,
		'publicly_queryable' => false,
		'show_ui'            => false,
		'show_in_menu'       => false,
		'query_var'          => true,
		'rewrite'            => array( 'slug' => 'ww_stats' ),
		'capability_type'    => 'post',
		'has_archive'        => true,
		'hierarchical'       => false,
		'menu_position'      => null,
		'supports'           => array( 'title' ),
		);

		register_post_type('ww_stats', $args);
	}

	public function grenerate_coupon( $amount ) {

		$coupon_code   = $this->generate_random_coupon_code(5); // Generating Random String
		$discount_type = 'percent'; // Type: fixed_cart, percent, fixed_product, percent_product
							
		$coupon = array(
		'post_title'    => wp_strip_all_tags($coupon_code),
		'post_content'  => '',
		'post_status'   => 'publish',
		'post_author'   => 1,
		'post_type'        => 'shop_coupon',
		);
							
		$new_coupon_id = wp_insert_post($coupon);

		$expiry_interval = woo_wheel_get_option('woo_wheel_settings_coupon_duration');
		$expiry_time     = woo_wheel_get_option('woo_wheel_settings_coupon_duration_time');
		$expiry_date     = gmdate('Y-m-d H:s:i', strtotime(' +' . $expiry_interval . ' ' . $expiry_time));
							
		// Add meta
		update_post_meta($new_coupon_id, 'discount_type', $discount_type);
		update_post_meta($new_coupon_id, 'coupon_amount', $amount);
		update_post_meta($new_coupon_id, 'individual_use', 'yes');
		update_post_meta($new_coupon_id, 'product_ids', '');
		update_post_meta($new_coupon_id, 'exclude_product_ids', '');
		update_post_meta($new_coupon_id, 'usage_limit', 1);
		update_post_meta($new_coupon_id, 'expiry_date', $expiry_date);
		update_post_meta($new_coupon_id, 'apply_before_tax', 'yes');
		update_post_meta($new_coupon_id, 'free_shipping', 'no');

		return $coupon_code;
	}

	public function generate_random_coupon_code( $length = 10 ) {
		$characters       = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$charactersLength = strlen($characters);
		$randomString     = '';
		for ($i = 0; $i < $length; $i++) {
			$randomString .= $characters[rand(0, $charactersLength - 1)];
		}
		return $randomString;
	}

	public function save_log() {

		check_ajax_referer('save_log', 'security');
		/**
		 * Action triggered before saving log for the WooCommerce Wheel of Fortune plugin.
		 *
		 * @since 1.0.0
		 */
		do_action('woo_wheel_before_saving_log');

		$resultText = '';
		if (isset($_POST['resultText']) ) {
			$resultText = sanitize_text_field(wp_unslash($_POST['resultText']));
		}

		$couponType = '';
		if (isset($_POST['couponType']) ) {
			$couponType = sanitize_text_field(wp_unslash($_POST['couponType']));
		}

		$coponValue = '';
		if (isset($_POST['coponValue']) ) {
			$coponValue = sanitize_text_field(wp_unslash($_POST['coponValue']));
		}

		$coupon_code = isset($_POST['couponCode']) ? sanitize_text_field(wp_unslash($_POST['couponCode'])) : '';

		if (isset($_POST['couponCode'])) {
			$coupon_code = sanitize_text_field(wp_unslash($_POST['couponCode']));
		} else {   
			$coupon_code = ''; 
		}

		// Create post object
		$wheel_log = array(
		'post_title' => wp_strip_all_tags($couponType),
		'post_content' => '',
		'post_status' => 'publish',
		'post_author' => get_current_user_id(),
		'post_type' => 'ww_stats',
		);
		
		// Insert the post into the database
		$log_id = wp_insert_post($wheel_log);

		$cart_url = wc_get_cart_url();
		$checkout_url = wc_get_checkout_url();
		$site_url = home_url();
		$is_option_enable = get_option('woo_wheel_settings_section_email_enable');

		$win = 'true';
		if ('no_prize' == $couponType ) {
			$win = 'false';

			$user_id = get_current_user_id();
			$user_data = get_userdata($user_id);

			$to = $user_data->user_email;
			$subject = 'Email Template - Lost';
			$message = get_option('woo_wheel_settings_email_template_lose_text');
			$message = str_replace('%username%', $user_data->user_login, $message);
			$headers = array( 'Content-Type: text/html; charset=UTF-8' );

			// Send email
			$no_prize = wp_mail($to, $subject, $message, $headers);

			if ($no_prize) {
				echo ' Lost email sent successfully!';
			}
		} else if ('coupon_auto' == $couponType ) {
			$user_id = get_current_user_id();
			$user_data = get_userdata($user_id);
			$to = $user_data->user_email;
			$prize = $coponValue;

			$message = get_option('woo_wheel_settings_email_template_win_text');
			$message = str_replace('%winning_text%', $prize, $message);
			$message = str_replace('%checkout_url%', $checkout_url, $message);
			$message = str_replace('%site_url%', $site_url, $message);
			$message = str_replace('%cart_url%', $cart_url, $message);
			$message = str_replace('%username%', $user_data->user_login, $message);
			$message = str_replace('%coupon%', $coupon_code, $message);
			/**
		   * Filter to make an optional to send an email after winning the auto generated coupon.
		   * 
		   * @param  bool  bool value.
		   * @return bool  it returns a bool value
		   * @since  1.8
		   */
			$send_email = apply_filters('woo_wheel_win_auto_coupon_email', true);
			if ( true == $send_email ) {
				$to = $user_data->user_email;
				$subject = 'Email Template - Win';
				$headers = array( 'Content-Type: text/html; charset=UTF-8' );
				// Send email
				$success = wp_mail($to, $subject, $message, $headers);
				if ($success) {
					echo ' winning email sent successfully!';
				}
			}
		} else if ('coupon' == $couponType ) {

			$user_id = get_current_user_id();
			$user_data = get_userdata($user_id);

			$prize = $coponValue;
			
			$coupon = new WC_Coupon($coponValue);
		
			$message = get_option('woo_wheel_settings_email_template_win_text');
			$message = str_replace('%winning_text%', $coupon->get_amount(), $message);
			$message = str_replace('%checkout_url%', $checkout_url, $message);
			$message = str_replace('%site_url%', $site_url, $message);
			$message = str_replace('%cart_url%', $cart_url, $message);
			$message = str_replace('%username%', $user_data->user_login, $message);
			$message = str_replace('%coupon%', $coupon_code, $message);
			/**
		   * Filter to make an optional to send an email after winning the simple coupon.
		   * 
		   * @param  bool  bool value.
		   * @return bool  it returns a bool value
		   * @since  1.8
		   */
			$send_email = apply_filters('woo_wheel_win_coupon_email', true);
			if ( true == $send_email ) {
				$to = $user_data->user_email;
				$subject = 'Email Template - Win';
				$headers = array( 'Content-Type: text/html; charset=UTF-8' );
				// Send email
				$success = wp_mail($to, $subject, $message, $headers);
				if ($success) {
					echo ' winning email sent successfully!';
				}
			}
		}
		
		update_post_meta($log_id, 'win', $win);
		update_post_meta($log_id, 'result_text', $resultText);
		update_post_meta($log_id, 'coupon_type', $couponType);
		update_post_meta($log_id, 'coupon_value', $coponValue);

		echo 'Log Saved!';

		die(); // this is required to terminate immediately and return a proper response
	}
	
	public function stats_table() {
		
		?>
		<style>
			button.woocommerce-save-button {
				display:none !important;
			}
		</style>
		<?php
		
		$paged = ( isset($_GET['paged']) ) ? intval($_GET['paged']) : 1;

		$date = '';
		if (isset($_GET['date']) ) {
			$date = sanitize_text_field(wp_unslash($_GET['date']));
		}

		$date_query = woo_wheel_get_date_query($date);
		
		$args       = array(
		'posts_per_page' => 10,
		'paged' => $paged,
		'post_type' => 'ww_stats',
		'post_status' => 'publish',
		'date_query' => $date_query,
		);
		$wheel_logs = new WP_Query($args);

		$actual_link = admin_url() . '/admin.php?page=wc-settings&tab=woo_wheel_settings&section=stats';

		$html = '<div class="ww-stats-filter"><a href="' . $actual_link . '&date=today" class="components-button editor-post-publish-panel__toggle is-button is-primary">Today</a><a href="' . $actual_link . '&date=thisweek" class="components-button editor-post-publish-panel__toggle is-button is-primary">This Week</a><a href="' . $actual_link . '&date=thismonth" class="components-button editor-post-publish-panel__toggle is-button is-primary">This Month</a><a href="' . $actual_link . '&date=thisyear" class="components-button editor-post-publish-panel__toggle is-button is-primary">This Year</a><a href="' . $actual_link . '&date=all" class="components-button editor-post-publish-panel__toggle is-button is-primary">All Data</a></div>';

		$html .= '<div class="ww-stats-circle">
				<div class="ww-cg-box">
					<div class="c100 p' . $this->get_winnnig_percentage($date_query) . '">
						<span>' . $this->get_winnnig_count($date_query) . '</span>
						<div class="slice">
							<div class="bar"></div>
							<div class="fill"></div>
						</div>
					</div>
					<div class="ww-box-title">' . __('NO OF WINS', 'woo-wheel') . '</div>
				</div>

				<div class="ww-cg-box">
					<div class="c100 p' . $this->get_lossing_percentage($date_query) . '">
						<span>' . $this->get_lossing_count($date_query) . '</span>
						<div class="slice">
							<div class="bar"></div>
							<div class="fill"></div>
						</div>
					</div>
					<div class="ww-box-title">' . __('NO OF LOSES', 'woo-wheel') . '</div>
				</div>

				<div class="ww-cg-box">
					<div class="c100 p100"><span>' . $this->get_total_no_of_play($date_query) . '</span>
						<div class="slice">
							<div class="bar"></div>
							<div class="fill"></div>
						</div>
					</div>
					<div class="ww-box-title">' . __('NO OF PLAYS', 'woo-wheel') . '</div>
				</div>
			</div>

			</div>';

		$html .= '<table class="ww_stats-tbl">';
		$html .= '<tr>
					<th>' . __('Username', 'woo-wheel') . '</th>
					<th>' . __('Type', 'woo-wheel') . '</th>
					<th>' . __('Winning', 'woo-wheel') . '</th>
					<th>' . __('Prize', 'woo-wheel') . '</th>
					<th>' . __('Date', 'woo-wheel') . '</th>
				</tr>';
		if ($wheel_logs->have_posts() ) {

			while ( $wheel_logs->have_posts() ) {
				$wheel_logs->the_post();

				$user_info = get_userdata($wheel_logs->post->post_author);
				$user_name = !empty(@$user_info->user_login) ? @$user_info->user_login : 'GUEST' ;

				$coupon_type = get_post_meta($wheel_logs->post->ID, 'coupon_type', true);
				$prize       = get_post_meta($wheel_logs->post->ID, 'coupon_value', true);
				$resultText  = get_post_meta($wheel_logs->post->ID, 'result_text', true);
				$post_date   = $wheel_logs->post->post_date;
				
				$html .= '<tr>
						<td>' . $user_name . '</td>
						<td>' . $coupon_type . '</td>
						<td>' . $resultText . '</td>
						<td>' . $prize . '</td>
						<td>' . $post_date . '</td>
					</tr>';
			}   
			wp_reset_postdata(); 
		}            
		$html .= '</table>';

		$html .= '<div class="spin-wheel-for-woocommerce-pagination">';
		
		$big = 999999999; // need an unlikely integer
		
		$html .= paginate_links(
			array(
			'base'         => admin_url() . 'admin.php?page=wc-settings&tab=woo_wheel_settings&paged=%#%',
			'total'        => $wheel_logs->max_num_pages,
			'current'      => get_query_var('paged'),
			'format'       => '?paged=%#%',
			'show_all'     => true,
			'type'         => 'list',
			'end_size'     => 2,
			'mid_size'     => 1,
			'prev_next'    => true,
			'prev_text'    => sprintf('<i></i> %1$s', __('Newer Posts', 'woo-wheel')),
			'next_text'    => sprintf('%1$s <i></i>', __('Older Posts', 'woo-wheel')),
			'add_args'     => false,
			'add_fragment' => '',
			) 
		);
 
		$html .= '</div>';

		$allowed_html = array(
			'div' => array(
				'class' => array(),
			),
			'a' => array(
				'href' => array(),
				'class' => array(),
			),
			'span' => array(),
			'table' => array(
				'class' => array(),
			),
			'th' => array(),
			'tr' => array(),
			'td' => array(),
		);

		echo wp_kses($html, $allowed_html);
	}

	/**
	 * Get winning count
	 */
	public function get_winnnig_count( $date ) {
		$args       = array(
		'posts_per_page'    => -1,
		'meta_key'          => 'win',
		'meta_value'        => 'true',
		'post_type'         => 'ww_stats',
		'post_status'       => 'publish', 
		'date_query'        => $date,
		);
		$wheel_logs = new WP_Query($args);

		return $wheel_logs->post_count;
	}

	/**
	 * Get winning Percentage value
	 */
	public function get_winnnig_percentage( $date ) {

		$value      = $this->get_winnnig_count($date);
		$total_play = $this->get_total_no_of_play($date);
		if (!empty($value) && !empty($total_play) ) {
			$value = $value / $total_play;
			$value = $value * 100;

			return round($value);
		}
	}

	/**
	 * Get lossing count
	 */
	public function get_lossing_count( $date ) {
		$args       = array(
		'posts_per_page'    => -1,
		'meta_key'          => 'win',
		'meta_value'        => 'false',
		'post_type'         => 'ww_stats',
		'post_status'       => 'publish',   
		'date_query'        => $date,         
		);
		$wheel_logs = new WP_Query($args);

		return $wheel_logs->post_count;
	}

	/**
	 * Get lossing Percentage value
	 */
	public function get_lossing_percentage( $date ) {

		$value      = $this->get_lossing_count($date);
		$total_play = $this->get_total_no_of_play($date);
		if (!empty($value) && !empty($total_play) ) {
			$value = $value / $total_play;
			$value = $value * 100;
		
			return round($value);
		}
	}

	/**
	 * Get total number of plays
	 */
	public function get_total_no_of_play( $date ) {
		$args       = array(
		'posts_per_page'    => -1,
		'post_type'         => 'ww_stats',
		'post_status'       => 'publish',
		'date_query'        => $date,            
		);
		$wheel_logs = new WP_Query($args);

		return $wheel_logs->post_count;
	}
}

$Woo_Wheel_Stats = new Woo_Wheel_Stats();
