<?php
if (! defined('ABSPATH') ) {
	exit; // Exit if accessed directly
}

/**
 * Class to view Wheel on frontend
 */
class Woo_Wheel_View {



	public function __construct() {
		add_action('wp_enqueue_scripts', array( $this, 'woo_wheel_styles_scripts' ));
		add_action('wp_footer', array( $this, 'apply_styles' ));
	}

	public function woo_wheel_styles_scripts() {

		//wp_register_script( 'woo-wheel-grunt', WOO_WHEEL_DIR . 'js/woo-wheel-grunt.js', array(), '1.4', true );
		wp_register_script('jquery', WOO_WHEEL_DIR . 'js/jquery.js', array(), '1.4', true);
		wp_register_script('Spin2WinWheel', WOO_WHEEL_DIR . 'js/Spin2WinWheel.js', array( 'jquery' ), '1.4', true);
		wp_register_script('ThrowPropsPlugin', WOO_WHEEL_DIR . 'js/ThrowPropsPlugin.min.js', array( 'jquery' ), '1.4', true);
		wp_register_script('SpinWheelIndex', WOO_WHEEL_DIR . 'js/index.js', array( 'jquery' ), '1.4', true);        

		wp_register_script('woo-wheel-snowfall', WOO_WHEEL_DIR . 'js/fallingsnow.js', array(), '1.4', true);
		$variables = array(
		'plugin_path' => WOO_WHEEL_ROOT_DIR_URL,
		);

		$variables = array(
		'plugin_path' => WOO_WHEEL_ROOT_DIR_URL,
		'winning_title' => woo_wheel_get_option('woo_wheel_settings_winning_title'),
		'winning_text_coupon' => is_user_logged_in() ? woo_wheel_get_option('woo_wheel_settings_winning_text_coupon') : esc_html__('Please login or create an account to claim your reward. <a href="' . esc_url(get_permalink(wc_get_page_id('myaccount'))) . '" style = "color: #d7e42d; text-decoration-line:underline;">Sign in</a>', 'woo-wheel'),
		'winning_text_link' => woo_wheel_get_option('woo_wheel_settings_winning_text_link'),
		'lossing_title' => woo_wheel_get_option('woo_wheel_settings_losing_title'),
		'lossing_text' => woo_wheel_get_option('woo_wheel_settings_losing_text'),
		'spin_done_text' => woo_wheel_get_option('woo_wheel_settings_done_text'),
		'spin_coupon_code' => get_option('woo_wheel_get_coupon_code'),
		'enable_cart_redirect' => esc_url(wc_get_cart_url()),
		'enable_cart_redirect_settings' => get_option('woo_wheel_settings_coupon_cart_redirect'),
		'ajax_url' => admin_url('admin-ajax.php'),
		'ajax_nonce' => wp_create_nonce('coupon_gen'),
		);
		wp_localize_script('Spin2WinWheel', 'woo_wheel_index', $variables);

		wp_enqueue_style('woo-wheel-style-font', WOO_WHEEL_DIR . 'css/g-fonts.css', array(), '1.6');
		wp_enqueue_style('woo-wheel-style', WOO_WHEEL_DIR . 'css/wheel-style.css', array(), '1.6');
		wp_enqueue_style('woo-wheel-front', WOO_WHEEL_DIR . 'css/woo-wheel-front.css', array(), '1.6');

		if (is_user_allowed_to_play() ) {
			//wp_enqueue_script( 'woo-wheel-grunt' );
			wp_enqueue_script('jquery');
			wp_enqueue_script('tweenmax', 'https://cdnjs.cloudflare.com/ajax/libs/gsap/2.1.3/TweenMax.min.js', array(), '1.6', true);
			wp_enqueue_script('tweenmax', 'https://cdnjs.cloudflare.com/ajax/libs/gsap/2.1.3/utils/Draggable.min.js', array(), '1.6', true);
			wp_enqueue_script('ThrowPropsPlugin');
			wp_enqueue_script('Spin2WinWheel');
			wp_enqueue_script('SpinWheelIndex');    
		}

		$values = array(
		'number_of_flakes'  => woo_wheel_get_option('woo_wheel_settings_snowfall_flakes_no'),
		'flakes_speed'      => woo_wheel_get_option('woo_wheel_settings_snowfall_flakes_speed'),
		);
		wp_localize_script('Spin2WinWheel', 'woo_wheel_snowfall', $values);

		if ('yes' == woo_wheel_get_option('woo_wheel_settings_enable_snowfall') && true == is_user_allowed_to_play() ) {
			wp_enqueue_script('woo-wheel-snowfall', WOO_WHEEL_DIR . 'js/fallingsnow.js', array(), '1.6', false, true);
		}
	}

	public function snowfall() {

		if ('yes' !=  woo_wheel_get_option('woo_wheel_settings_enable_snowfall') ) {
			return;
		}

		$allowed_html = array(
		'div' => array(
		'id' => 'snowflakeContainer',
		),
		'p' => array(
		'class' => 'snowflake',
		),
		);

		$html = '<div id="snowflakeContainer">
					<p class="snowflake">*</p>
					<p class="snowflake">*</p>
					<p class="snowflake">*</p>
				</div>';

		echo wp_kses($html, $allowed_html);
	}

	public function woo_wheel_html() {
		global $wp;
		$scroll_trigger_by_class = ( !empty(get_option('woo_wheel_settings_scroll_trigger_by_class')) ) ? get_option('woo_wheel_settings_scroll_trigger_by_class') : '' ;
		?>
			<div id="wheel-wrapper" class="<?php echo esc_attr($scroll_trigger_by_class); ?>" style="display:<?php echo esc_attr($scroll_trigger_by_class ? 'block' : 'none'); ?>;">


		<div class="ww-close-popup">X</div>
	
		<div class="ww-wrapper">
			<div class="ww-body">
		<?php
		if (!empty(woo_wheel_get_option('woo_wheel_logo')) ) {
			?>
				<div class="ww-logo"><img src="<?php echo esc_html(woo_wheel_get_option('woo_wheel_logo')); ?>" /></div>
		<?php } ?>
			<div class="wheel-popup-details">
				<div class="ww-details">

				
					<div class="ww-title"> <?php echo esc_html(woo_wheel_get_option('woo_wheel_popup_title')); ?> </div>
					<div class="ww-explainer"> <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_explainer_text')); ?> </div>
					<div class="spinButton">
						<button class="spinBtn"> <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_spin_button')); ?> </button>
					</div>
					<div class="ww-disclaimer"> <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_disclaimer_text')); ?> </div>

				</div>
				
				<div class="toast">
				  <p class="ww-toast-title">HURRAY! YOUVE WON A PRIZE. LUCKY YOU!</p>
				</div>
			</div>
			</div>

			<div class="wheelContainer">
			<svg class="wheelSVG" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" text-rendering="optimizeSpeed">
				<defs>
					<filter id="shadow" x="-100%" y="-100%" width="550%" height="550%">
						<feOffset in="SourceAlpha" dx="0" dy="0" result="offsetOut"></feOffset>
						<feGaussianBlur stdDeviation="9" in="offsetOut" result="drop" />
						<feColorMatrix in="drop" result="color-out" type="matrix" values="0 0 0 0   0
				0 0 0 0   0 
				0 0 0 0   0 
				0 0 0 .3 0" />
						<feBlend in="SourceGraphic" in2="color-out" mode="normal" />
					</filter>
				</defs>
				<g class="mainContainer">
					<g class="wheel">                         
					</g>
				</g>
				<g class="centerCircle" />
				<g class="wheelOutline" />
				<g class="pegContainer" opacity="1">
					<path class="peg" fill="#EEEEEE" d="M22.139,0C5.623,0-1.523,15.572,0.269,27.037c3.392,21.707,21.87,42.232,21.87,42.232 s18.478-20.525,21.87-42.232C45.801,15.572,38.623,0,22.139,0z" />
				</g>
				<g class="valueContainer" />
		<?php
		if (!empty(woo_wheel_get_option('woo_wheel_wheel_center_logo')) ) {
			?>
					<image xlink:href="<?php echo esc_url(woo_wheel_get_option('woo_wheel_wheel_center_logo')); ?>" width="200" height="200" x="412" y="284"></image>
		<?php } ?>
			</svg>
	  
		</div>
	</div>

		<?php
		echo '<div class="privacy-policy-page"><a href="' . esc_url(get_permalink(get_option('woo_wheel_settings_privacy_dropdown'))) . '" target="_blank">' . esc_html(get_option('woo_wheel_settings_privacy_title')) . '</a></div>';
		?>

		
	</div>
		<?php
	}

	public function apply_styles() {

		?>
		<style>
			#wheel-wrapper{
				background-color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_background_color')); ?>;
			}

			.wheel-popup-details .ww-title {
				color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_heading_color')); ?>;
			}

			.wheel-popup-details .ww-explainer, .wheel-popup-details .ww-disclaimer {
				color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_text_color')); ?>;
			}

			#wheel-wrapper button.spinBtn {
				background-color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_button_background')); ?>;
				color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_button_text_color')); ?>;
			}
			.play-end button {
				background-color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_button_background')); ?>;
				color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_button_text_color')); ?>;
			}

			.toast {
				background-color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_winning_text_bgcolor')); ?>;
				color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_winning_text_color')); ?>;
			}
			.toast .ww-toast-title {
				color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_winning_text_color')); ?>;
			}

			.toast .ww-toast-text {
				color: <?php echo esc_html(woo_wheel_get_option('woo_wheel_settings_winning_text_color')); ?>;
			}
		</style>
		<?php
	}
}

$Woo_Wheel_View = new Woo_Wheel_View();
