<?php
if (! defined('ABSPATH') ) {
	exit; // Exit if accessed directly
}

/**
 * Getting option values from Database
 * 
 * @param   $doption_nameate (string) option key which is used in WordPress option_table
 * @since   1.0
 * @version 1.0
 */
function woo_wheel_get_option( $option_name ) {
	if (isset($option_name) && !empty($option_name) ) {
		return get_option($option_name);
	}
}

/**
 * Getting the list if all available Coupons
 * 
 * @since   1.0
 * @version 1.0
 */
function woo_wheel_getAvailableCoupons() {
	$args = array(
	'posts_per_page'   => -1,
	'orderby'          => 'title',
	'order'            => 'asc',
	'post_type'        => 'shop_coupon',
	'post_status'      => 'publish',
	);
		   
	$coupons = get_posts($args);

	return $coupons;
}

/**
 * Function to get how many times a user play a wheel
 * 
 * @param   $user_id (int) WordPress user id
 * @param   $date    (string) date
 * @since   1.0
 * @version 1.0
 */
function woo_wheel_get_number_of_plays( $user_id, $date = 'today' ) {

	$args = array(
	'posts_per_page'    => -1,
	'post_type'         => 'ww_stats',
	'post_status'       => 'publish',
	'author'            => $user_id,
	'date_query'        => woo_wheel_get_date_query($date),         
	);
	
	$wheel_logs = new WP_Query($args);
	return $wheel_logs->post_count;
}

/**
 * Count Wheel Slices
 * 
 * @since   1.0
 * @version 1.0
 */
function woo_wheel_get_slices_count() {
	$no_wheel_slices  = 0;
	$woo_wheel_slices = woo_wheel_get_option('woo_wheel_slices');
	if (isset($woo_wheel_slices['woo_wheel_slices']['coupon_type']) ) {
		$no_wheel_slices = count($woo_wheel_slices['woo_wheel_slices']['coupon_type']);    
	}
	return $no_wheel_slices;
}

/**
 * Get Date Query
 * 
 * @param   $date (string) string date (thisweek, thismonth, thisyear, today)
 * @since   1.0
 * @version 1.0
 */
function woo_wheel_get_date_query( $date ) {
	$date_query = array();
	if (isset($date) && 'thisweek' == $date ) {
		$date_query = array(
		array(
		'year' => gmdate('Y'),
		'week' => gmdate('W'),
		),
		);
	} elseif (isset($date) && 'thismonth' == $date ) {
		$today      = getdate();
		$date_query = array(
		array(
		'month' => $today['mon'],
		),
		);
	} elseif (isset($date) && 'thisyear' == $date ) {
		$today      = getdate();
		$date_query = array(
		array(
		'year' => $today['year'],
		),
		);
	} elseif (isset($date) && 'today' == $date ) {
		$today      = getdate();
		$date_query = array(
		array(
		'year' => $today['year'],
		'month' => $today['mon'],
		'day' => $today['mday'],
		),
		);
	}

	return $date_query;
}

/**
 * Count user's post of last x number of days
 * 
 * @param   $num_of_days (int) num of days to get count of the post
 * @param   $user_id     (int) user ID to specify the user
 * @since   1.0
 * @version 1.0
 */
function woo_wheel_get_user_post_count_by_days( $num_of_days, $user_id ) {
	$args = array(
	'posts_per_page'    => -1,
	'post_type'         => 'ww_stats',
	'post_status'       => 'publish',
	'author'            => $user_id,
	'date_query' => array(
	'after' => gmdate('Y-m-d', strtotime('-' . $num_of_days . ' days')), 
	),         
	);

	$wheel_logs = new WP_Query($args);
	return $wheel_logs->post_count;
}

/**
 * Check whether a coupon is valid or not
 * 
 * @param   $coupon (string) coupon code
 * @since   1.0
 * @version 1.0
 */
function woo_wheel_is_coupon_valid( $coupon ) {
	$coupon         = new WC_Coupon($coupon);   
	$discounts      = new WC_Discounts(WC()->cart);
	$valid_response = $discounts->is_coupon_valid($coupon);
	if (is_wp_error($valid_response) ) {
		return false;
	} else {
		return true;
	}
}

/**
 * Check if user is allowed to play a spin
 * 
 * @param   $user_id (int) worpdress user id
 * @since   1.0
 * @version 1.0
 */
function is_user_allowed_to_play( $user_id = 0 ) {

	if (empty($user_id) ) {
		$user_id = get_current_user_id();
	}

	$occurance_delay = woo_wheel_get_option('woo_wheel_settings_occurance_delay');

	if ($occurance_delay > 0 ) {
		$played = woo_wheel_get_user_post_count_by_days($occurance_delay, $user_id);
		if (!empty($played) ) {
			return false;
		}
	}

	$wheel_date = woo_wheel_get_option('woo_wheel_limited_date');
	if (!empty($wheel_date) ) {
		$start_date = '';
		$end_date   = '';
		if (!empty($wheel_date['wheel_start_date']['date']) ) {
			$start_date = $wheel_date['wheel_start_date']['date'] . ' ' . $wheel_date['wheel_start_date']['hour'] . ':' . $wheel_date['wheel_start_date']['minute'] . ' ' . $wheel_date['wheel_start_date']['time_format'];
		}
		if (!empty($wheel_date['wheel_end_date']['date']) ) {
			$end_date = $wheel_date['wheel_end_date']['date'] . ' ' . $wheel_date['wheel_end_date']['hour'] . ':' . $wheel_date['wheel_end_date']['minute'] . ' ' . $wheel_date['wheel_end_date']['time_format'];
		}
		return wheel_display_period($start_date, $end_date);
	}

	return true;
}

/**
 * Check if wheel is allow to show on pages
 * 
 * @since   1.0
 * @version 1.0
 */
function is_wheel_allow_on_pages_posts( $page_id = 0 ) {

	if (empty($page_id) ) {
		$current_post_id = get_the_ID();
	}

	$current_post_type = get_post_type($current_post_id);
	$current_post_title = get_the_title($current_post_id);
	if ('all_pages' == woo_wheel_get_option('woo_wheel_settings_on_pages') && ( 'page' == $current_post_type || is_shop() ) ) {
		return true;
	}

	if ('all_posts' == woo_wheel_get_option('woo_wheel_settings_on_posts') && 'post' == $current_post_type && !is_home() ) {
		return true;
	}

	$selected_pages = woo_wheel_get_option('woo_wheel_settings_select_pages');
	$selected_posts = woo_wheel_get_option('woo_wheel_settings_select_posts');

	if (is_shop() ) {
		if (is_array($selected_pages) ) {
			if (in_array(wc_get_page_id('shop'), $selected_pages) && woo_wheel_get_option('woo_wheel_settings_on_pages') == 'specific_pages' ) {
				return true;
			}
		}
		
	}

	if (( woo_wheel_get_option('woo_wheel_settings_on_pages') == 'specific_pages' && in_array($current_post_id, $selected_pages) ) || ( woo_wheel_get_option('woo_wheel_settings_on_posts') == 'specific_posts' && in_array($current_post_title, $selected_posts) ) ) {
		return true;
	}
	
	return false;
}

/**
 * Display wheel for the limited time
 * 
 * @since   1.4
 * @version 1.0
 */
function wheel_display_period( $start_date = '', $end_date = '' ) {
	
	$enable_limited_time = woo_wheel_get_option('woo_wheel_enable_limited_time');
	
	if (( empty($start_date) || !isset($start_date) ) || 'no' == $enable_limited_time ) {
		return true;
	}
	
	$current_date = gmdate('F d, Y h:i a');
	$current_date = strtotime($current_date);

	$start_date = strtotime($start_date);
	$end_date   = strtotime($end_date);

	if ($current_date > $start_date && ( $current_date < $end_date || '' == $end_date ) ) {
		return true;
	}

	return false;
}
